within MoMoLib.Air;
partial package Air_Base "Properties of dry air calculated using the equation of state by Lemmon et. al."


extends Modelica.Media.Interfaces.PartialPureSubstance(
  mediumName="Air",
  substanceNames={"air"},
  singleState=false,
  SpecificEnthalpy(start=1.0e5, nominal=5.0e5),
  Density(start=1.0, nominal=1.2),
  AbsolutePressure(
    start=1e5,
    nominal=1e5,
    min=1.0,
    max=2000e6),
  Temperature(
    start=273.15,
    nominal=293.15,
    min=130,
    max=2000));

constant Boolean ph_explicit
  "true if explicit in pressure and specific enthalpy";
constant Boolean dT_explicit "true if explicit in density and temperature";
constant Boolean pT_explicit "true if explicit in pressure and temperature";


  redeclare record extends ThermodynamicState "thermodynamic state"
  SpecificEnthalpy h "specific enthalpy";
  Density d "density";
  Temperature T "temperature";
  AbsolutePressure p "pressure";
  end ThermodynamicState;


  redeclare replaceable model extends BaseProperties(
  h(stateSelect=if ph_explicit and preferredMediumStates then StateSelect.prefer
         else StateSelect.default),
  d(stateSelect=if dT_explicit and preferredMediumStates then StateSelect.prefer
         else StateSelect.default),
  T(stateSelect=if (pT_explicit or dT_explicit) and preferredMediumStates then
        StateSelect.prefer else StateSelect.default),
  p(stateSelect=if (pT_explicit or ph_explicit) and preferredMediumStates then
        StateSelect.prefer else StateSelect.default))
  "Base properties of water"

  equation
  MM = Constants.MM;
  if dT_explicit then
    p = pressure_dT(d, T);
    h = specificEnthalpy_dT(d, T);
  elseif ph_explicit then
    d = density_ph(p, h);
    T = temperature_ph(p, h);
  else
    h = specificEnthalpy_pT(p, T);
    d = density_pT(p, T);
  end if;
  u = h - p/d;
  R = Constants.R;
  h = state.h;
  p = state.p;
  T = state.T;
  d = state.d;
  end BaseProperties;


  redeclare function density_ph
  "Computes density as a function of pressure and specific enthalpy"
  extends Modelica.Icons.Function;
  input AbsolutePressure p "Pressure";
  input SpecificEnthalpy h "Specific enthalpy";
  output Density d "Density";
  algorithm
  d := Air_Utilities.rho_ph(p, h);
  end density_ph;


  redeclare function temperature_ph
  "Computes temperature as a function of pressure and specific enthalpy"
  extends Modelica.Icons.Function;
  input AbsolutePressure p "Pressure";
  input SpecificEnthalpy h "Specific enthalpy";
  output Temperature T "Temperature";
  algorithm
  T := Air_Utilities.T_ph(p, h);
  end temperature_ph;


  redeclare function temperature_ps
  "Compute temperature from pressure and specific enthalpy"
  extends Modelica.Icons.Function;
  input AbsolutePressure p "Pressure";
  input SpecificEntropy s "Specific entropy";
  output Temperature T "Temperature";
  algorithm
  T := Air_Utilities.T_ps(p, s);
  end temperature_ps;


  redeclare function density_ps
  "Computes density as a function of pressure and specific enthalpy"
  extends Modelica.Icons.Function;
  input AbsolutePressure p "Pressure";
  input SpecificEntropy s "Specific entropy";
  output Density d "density";
  algorithm
  d := Air_Utilities.rho_ps(p, s);
  end density_ps;


  redeclare function pressure_dT
  "Computes pressure as a function of density and temperature"
  extends Modelica.Icons.Function;
  input Density d "Density";
  input Temperature T "Temperature";
  output AbsolutePressure p "Pressure";
  algorithm
  p := Air_Utilities.p_dT(d, T);
  end pressure_dT;


  redeclare function specificEnthalpy_dT
  "Computes specific enthalpy as a function of density and temperature"
  extends Modelica.Icons.Function;
  input Density d "Density";
  input Temperature T "Temperature";
  output SpecificEnthalpy h "specific enthalpy";
  algorithm
  h := Air_Utilities.h_dT(d, T);
  end specificEnthalpy_dT;


  redeclare function specificEnthalpy_pT
  "Computes specific enthalpy as a function of pressure and temperature"
  extends Modelica.Icons.Function;
  input AbsolutePressure p "Pressure";
  input Temperature T "Temperature";
  output SpecificEnthalpy h "specific enthalpy";
  algorithm
  h := Air_Utilities.h_pT(p, T);
  end specificEnthalpy_pT;


  redeclare function specificEnthalpy_ps
  "Computes specific enthalpy as a function of pressure and temperature"
  extends Modelica.Icons.Function;
  input AbsolutePressure p "Pressure";
  input SpecificEntropy s "Specific entropy";
  output SpecificEnthalpy h "specific enthalpy";
  algorithm
  h := Air_Utilities.h_ps(p, s);
  end specificEnthalpy_ps;


  redeclare function density_pT
  "Computes density as a function of pressure and temperature"
  extends Modelica.Icons.Function;
  input AbsolutePressure p "Pressure";
  input Temperature T "Temperature";
  output Density d "Density";
  algorithm
  d := Air_Utilities.rho_pT(p, T);
  end density_pT;


  redeclare function extends dynamicViscosity
  "Return dynamic viscosity as a function of the thermodynamic state record"
  algorithm
  eta := Air_Utilities.dynamicViscosity(state);
  end dynamicViscosity;


  redeclare function extends thermalConductivity
  "Thermal conductivity of water"
  algorithm
  lambda := Air_Utilities.thermalConductivity(state);
  end thermalConductivity;


  redeclare function extends pressure "return pressure of ideal gas"
  algorithm
  p := state.p;
  end pressure;


  redeclare function extends temperature "return temperature of ideal gas"
  algorithm
  T := state.T;
  end temperature;


  redeclare function extends density "return density of ideal gas"
  algorithm
  d := state.d;
  end density;


  redeclare function extends specificEnthalpy "Return specific enthalpy"
  algorithm
  h := state.h;
  end specificEnthalpy;


  redeclare function extends specificInternalEnergy
  "Return specific internal energy"
  algorithm
  u := state.h - state.p/state.d;
  end specificInternalEnergy;


  redeclare function extends specificGibbsEnergy "Return specific Gibbs energy"
  algorithm
  g := state.h - state.T*specificEntropy(state);
  end specificGibbsEnergy;


  redeclare function extends specificHelmholtzEnergy
  "Return specific Helmholtz energy"
  algorithm
  f := state.h - state.p/state.d - state.T*specificEntropy(state);
  end specificHelmholtzEnergy;


  redeclare function extends specificEntropy "specific entropy of water"
  algorithm
  if dT_explicit then
    s := Air_Utilities.s_dT(state.d, state.T);
  elseif pT_explicit then
    s := Air_Utilities.s_pT(state.p, state.T);
  else
    s := Air_Utilities.s_ph(state.p, state.h);
  end if;
  end specificEntropy;


  redeclare function extends specificHeatCapacityCp
  "specific heat capacity at constant pressure of water"

  algorithm
  if dT_explicit then
    cp := Air_Utilities.cp_dT(state.d, state.T);
  elseif pT_explicit then
    cp := Air_Utilities.cp_pT(state.p, state.T);
  else
    cp := Air_Utilities.cp_ph(state.p, state.h);
  end if;
  end specificHeatCapacityCp;


  redeclare function extends specificHeatCapacityCv
  "specific heat capacity at constant volume of water"
  algorithm
  if dT_explicit then
    cv := Air_Utilities.cv_dT(state.d, state.T);
  elseif pT_explicit then
    cv := Air_Utilities.cv_pT(state.p, state.T);
  else
    cv := Air_Utilities.cv_ph(state.p, state.h);
  end if;
  end specificHeatCapacityCv;


  redeclare function extends isentropicExponent "Return isentropic exponent"
  algorithm
  if dT_explicit then
    gamma := Air_Utilities.isentropicExponent_dT(state.d, state.T);
  elseif pT_explicit then
    gamma := Air_Utilities.isentropicExponent_pT(state.p, state.T);
  else
    gamma := Air_Utilities.isentropicExponent_ph(state.p, state.h);
  end if;
  end isentropicExponent;


  redeclare function extends isothermalCompressibility
  "Isothermal compressibility of water"
  algorithm
  if dT_explicit then
    kappa := Air_Utilities.kappa_dT(state.d, state.T);
  elseif pT_explicit then
    kappa := Air_Utilities.kappa_pT(state.p, state.T);
  else
    kappa := Air_Utilities.kappa_ph(state.p, state.h);
  end if;
  end isothermalCompressibility;


  redeclare function extends isobaricExpansionCoefficient
  "isobaric expansion coefficient of water"
  algorithm
  if dT_explicit then
    beta := Air_Utilities.beta_dT(state.d, state.T);
  elseif pT_explicit then
    beta := Air_Utilities.beta_pT(state.p, state.T);
  else
    beta := Air_Utilities.beta_ph(state.p, state.h);
  end if;
  end isobaricExpansionCoefficient;


  redeclare function extends velocityOfSound
  "Return velocity of sound as a function of the thermodynamic state record"
  algorithm
  if dT_explicit then
    a := Air_Utilities.velocityOfSound_dT(state.d, state.T);
  elseif pT_explicit then
    a := Air_Utilities.velocityOfSound_pT(state.p, state.T);
  else
    a := Air_Utilities.velocityOfSound_ph(state.p, state.h);
  end if;
  end velocityOfSound;


  redeclare function extends density_derh_p
  "density derivative by specific enthalpy"
  algorithm
  ddhp := Air_Utilities.ddhp(state.p, state.h);
  end density_derh_p;


  redeclare function extends density_derp_h "density derivative by pressure"
  algorithm
  ddph := Air_Utilities.ddph(state.p, state.h);
  end density_derp_h;

//   redeclare function extends density_derT_p
//     "density derivative by temperature"
//   algorithm
//     ddTp := IF97_Utilities.ddTp(state.p, state.h, state.phase);
//   end density_derT_p;
//
//   redeclare function extends density_derp_T
//     "density derivative by pressure"
//   algorithm
//     ddpT := IF97_Utilities.ddpT(state.p, state.h, state.phase);
//   end density_derp_T;


  redeclare function extends setState_dTX
  "Return thermodynamic state of water as function of d and T"
  algorithm
  state := ThermodynamicState(
      d=d,
      T=T,
      h=specificEnthalpy_dT(d, T),
      p=pressure_dT(d, T));
  end setState_dTX;


  redeclare function extends setState_phX
  "Return thermodynamic state of water as function of p and h"
  algorithm
  state := ThermodynamicState(
      d=density_ph(p, h),
      T=temperature_ph(p, h),
      h=h,
      p=p);
  end setState_phX;


  redeclare function extends setState_psX
  "Return thermodynamic state of water as function of p and s"
  algorithm
  state := ThermodynamicState(
      d=density_ps(p, s),
      T=temperature_ps(p, s),
      h=specificEnthalpy_ps(p, s),
      p=p);
  end setState_psX;


  redeclare function extends setState_pTX
  "Return thermodynamic state of water as function of p and T"
  algorithm
  state := ThermodynamicState(
      d=density_pT(p, T),
      T=T,
      h=specificEnthalpy_pT(p, T),
      p=p);
  end setState_pTX;


  redeclare function extends setSmoothState
  "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
  import Modelica.Media.Common.smoothStep;
  algorithm
  state := ThermodynamicState(
      p=smoothStep(
        x,
        state_a.p,
        state_b.p,
        x_small),
      h=smoothStep(
        x,
        state_a.h,
        state_b.h,
        x_small),
      d=density_ph(smoothStep(
        x,
        state_a.p,
        state_b.p,
        x_small), smoothStep(
        x,
        state_a.h,
        state_b.h,
        x_small)),
      T=temperature_ph(smoothStep(
        x,
        state_a.p,
        state_b.p,
        x_small), smoothStep(
        x,
        state_a.h,
        state_b.h,
        x_small)));
  end setSmoothState;


annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{-100,-100},
          {100,100}}), graphics={Text(
        extent={{-94,84},{94,40}},
        lineColor={127,191,255},
        textString="IF97"), Text(
        extent={{-94,20},{94,-24}},
        lineColor={127,191,255},
        textString="water")}), Documentation(info="<HTML>
<p>
This model calculates medium properties
for water in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions
according to the IAPWS/IF97 standard, i.e., the accepted industrial standard
and best compromise between accuracy and computation time.
For more details see <a href=\"modelica://Modelica.Media.Water.IF97_Utilities\">
Modelica.Media.Water.IF97_Utilities</a>. Three variable pairs can be the
independent variables of the model:
</p>
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are the most natural choice for general applications. This is the recommended choice for most general purpose applications, in particular for power plants.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural choice for applications where water is always in the same phase, both for liquid water and steam.</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit variables of the Helmholtz function in the near-critical region and can be the best choice for applications with super-critical or near-critial states.</li>
</ol>
<p>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a> and in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>
<p>Many further properties can be computed. Using the well-known Bridgman's Tables, all first partial derivatives of the standard thermodynamic variables can be computed easily.</p>
</html>"));
end Air_Base;
