within MoMoLib.Air.Air_Utilities.Basic;
function Helmholtz "Helmholtz equation of state"
  extends Modelica.Icons.Function;
  input SI.Density d "density";
  input SI.Temperature T "temperature (K)";
  output Modelica.Media.Common.HelmholtzDerivs f
    "dimensionless Helmholtz function and dervatives w.r.t. delta and tau";

protected
  final constant Real[13] N_0={0.605719400E-007,-0.210274769E-004,-0.158860716E-003,
      -0.13841928076E002,0.17275266575E002,-0.195363420E-003,0.2490888032E001,
      0.791309509,0.212236768,0.197938904,0.2536365E002,0.1690741E002,
      0.8731279E002};
  final constant Real[19] N={0.118160747229,0.713116392079,-0.161824192067E001,
      0.714140178971E-001,-0.865421396646E-001,0.134211176704,
      0.112626704218E-001,-0.420533228842E-001,0.349008431982E-001,
      0.164957183186E-003,-0.101365037912,-0.173813690970,-0.472103183731E-001,
      -0.122523554253E-001,-0.146629609713,-0.316055879821E-001,
      0.233594806142E-003,0.148287891978E-001,-0.938782884667E-002};
  final constant Integer[19] i={1,1,1,2,3,3,4,4,4,6,1,3,5,6,1,3,11,1,3};
  final constant Real[19] j={0,0.33,1.01,0,0,0.15,0,0.2,0.35,1.35,1.6,0.8,0.95,
      1.25,3.6,6,3.25,3.5,15};
  final constant Integer[19] l={0,0,0,0,0,0,0,0,0,0,1,1,1,1,2,2,2,3,3};

algorithm
  f.d := d;
  f.T := T;
  f.R := Air_Base.Constants.R;
  //Reduced density
  f.delta := d/(Air_Base.Constants.MM*Air_Base.Constants.rhored);
  //Reciprocal reduced temperature
  f.tau := Air_Base.Constants.Tred/T;

  //Dimensionless Helmholtz equation
  f.f := 0;
  //Ideal-gas part
  for k in 1:5 loop
    f.f := f.f + N_0[k]*f.tau^(k - 4);
  end for;
  f.f := f.f + log(f.delta) + N_0[6]*f.tau*sqrt(f.tau) + N_0[7]*log(f.tau) +
    N_0[8]*log(1 - exp(-N_0[11]*f.tau)) + N_0[9]*log(1 - exp(-N_0[12]*f.tau))
     + N_0[10]*log(2/3 + exp(N_0[13]*f.tau));
  //Residual part
  for k in 1:10 loop
    f.f := f.f + N[k]*f.delta^i[k]*f.tau^j[k];
  end for;
  for k in 11:19 loop
    f.f := f.f + N[k]*f.delta^i[k]*f.tau^j[k]*exp(-f.delta^l[k]);
  end for;

  //First derivative of f w.r.t. delta
  f.fdelta := 0;
  //Ideal-gas part
  f.fdelta := 1/f.delta;
  //Residual part
  for k in 1:10 loop
    f.fdelta := f.fdelta + i[k]*N[k]*f.delta^(i[k] - 1)*f.tau^j[k];
  end for;
  for k in 11:19 loop
    f.fdelta := f.fdelta + N[k]*f.delta^(i[k] - 1)*f.tau^j[k]*exp(-f.delta^l[k])
      *(i[k] - l[k]*f.delta^l[k]);
  end for;

  //Second derivative of f w.r.t. delta
  f.fdeltadelta := 0;
  //Ideal-gas part
  f.fdeltadelta := -1/f.delta^2;
  //Residual part
  for k in 1:10 loop
    f.fdeltadelta := f.fdeltadelta + i[k]*(i[k] - 1)*N[k]*f.delta^(i[k] - 2)*f.tau
      ^j[k];
  end for;
  for k in 11:19 loop
    f.fdeltadelta := f.fdeltadelta + N[k]*f.delta^(i[k] - 2)*f.tau^j[k]*exp(-f.delta
      ^l[k])*((i[k] - l[k]*f.delta^l[k])*(i[k] - 1 - l[k]*f.delta^l[k]) - l[k]^
      2*f.delta^l[k]);
  end for;

  //First derivative of f w.r.t. tau
  f.ftau := 0;
  //Ideal-gas part
  for k in 1:5 loop
    f.ftau := f.ftau + (k - 4)*N_0[k]*f.tau^(k - 5);
  end for;
  f.ftau := f.ftau + 1.5*N_0[6]*sqrt(f.tau) + N_0[7]/f.tau + N_0[8]*N_0[11]/(
    exp(N_0[11]*f.tau) - 1) + N_0[9]*N_0[12]/(exp(N_0[12]*f.tau) - 1) + N_0[10]
    *N_0[13]/(2/3*exp(-N_0[13]*f.tau) + 1);
  //Residual part
  for k in 1:10 loop
    f.ftau := f.ftau + j[k]*N[k]*f.delta^i[k]*f.tau^(j[k] - 1);
  end for;
  for k in 11:19 loop
    f.ftau := f.ftau + j[k]*N[k]*f.delta^i[k]*f.tau^(j[k] - 1)*exp(-f.delta^l[k]);
  end for;

  //Second derivative of f w.r.t. tau
  f.ftautau := 0;
  //Ideal-gas part
  for k in 1:3 loop
    f.ftautau := f.ftautau + (k - 4)*(k - 5)*N_0[k]*f.tau^(k - 6);
  end for;
  f.ftautau := f.ftautau + 0.75*N_0[6]/sqrt(f.tau) - N_0[7]/f.tau^2 - N_0[8]*
    N_0[11]^2*exp(N_0[11]*f.tau)/(exp(N_0[11]*f.tau) - 1)^2 - N_0[9]*N_0[12]^2*
    exp(N_0[12]*f.tau)/(exp(N_0[12]*f.tau) - 1)^2 + 2/3*N_0[10]*N_0[13]^2*exp(-
    N_0[13]*f.tau)/(2/3*exp(-N_0[13]*f.tau) + 1)^2;
  //Residual part
  for k in 1:10 loop
    f.ftautau := f.ftautau + j[k]*(j[k] - 1)*N[k]*f.delta^i[k]*f.tau^(j[k] - 2);
  end for;
  for k in 11:19 loop
    f.ftautau := f.ftautau + j[k]*(j[k] - 1)*N[k]*f.delta^i[k]*f.tau^(j[k] - 2)
      *exp(-f.delta^l[k]);
  end for;

  //Mixed derivative of f w.r.t. delta and tau
  f.fdeltatau := 0;
  //Residual part (Ideal-gas part is zero)
  for k in 1:10 loop
    f.fdeltatau := f.fdeltatau + i[k]*j[k]*N[k]*f.delta^(i[k] - 1)*f.tau^(j[k]
       - 1);
  end for;
  for k in 11:19 loop
    f.fdeltatau := f.fdeltatau + j[k]*N[k]*f.delta^(i[k] - 1)*f.tau^(j[k] - 1)*
      exp(-f.delta^l[k])*(i[k] - l[k]*f.delta^l[k]);
  end for;

end Helmholtz;
