within MoMoLib.Air.Air_Utilities.Transport;
function lambda_dT "Return thermal conductivity as a function of d and T"
  extends Modelica.Icons.Function;
  input SI.Density d "Density";
  input SI.Temperature T "Temperature";
  output SI.ThermalConductivity lambda "Thermal conductivity";

protected
  Modelica.Media.Common.HelmholtzDerivs f
    "dimensionless Helmholtz function and dervatives w.r.t. delta and tau";
  SI.ThermalConductivity lambda_0=0 "Dilute gas thermal conductivity";
  SI.ThermalConductivity lambda_r=0 "Residual fluid thermal conductivity";
  SI.ThermalConductivity lambda_c=0 "Thermal conductivity critical enhancement";
  Real Omega "Collision integral";
  SI.DynamicViscosity eta_0=0 "Dilute gas viscosity";
  Real pddT;
  Real pddTref;
  Real pdTp;
  Real xi;
  Real xiref;
  Real Omega_tilde;
  Real Omega_0_tilde;
  Real cv;
  Real cp;
  final constant Real[5] b={0.431,-0.4623,0.08406,0.005341,-0.00331};
  final constant Real[9] Ncon={1.308,1.405,-1.036,8.743,14.76,-16.62,3.793,-6.142,
      -0.3778};
  final constant Real[9] tcon={0.0,-1.1,-0.3,0.1,0.0,0.5,2.7,0.3,1.3};
  final constant Integer[9] dcon={0,0,0,1,2,3,7,7,11};
  final constant Integer[9] lcon={0,0,0,0,0,2,2,2,2};
  final constant Integer[9] gammacon={0,0,0,0,0,1,1,1,1};

algorithm
  //chi_tilde in at the reference temperature 265.262
  f := Basic.Helmholtz(d, 265.262);
  pddTref := Air_Base.Constants.R_bar*265.262*(1 + 2*f.delta*(f.fdelta - 1/f.delta)
     + f.delta^2*(f.fdeltadelta + 1/f.delta^2));
  xiref := Air_Base.Constants.pred*(d/Air_Base.Constants.MM)/Air_Base.Constants.rhored
    ^2/pddTref;
  //calculating f at the given state
  f := Basic.Helmholtz(d, T);
  Omega := exp(
    Modelica.Media.Incompressible.TableBased.Polynomials_Temp.evaluate({b[5],b[
    4],b[3],b[2],b[1]}, log(T/103.3)));
  //Ideal-gas part of dynamic viscosity
  eta_0 := 0.0266958*sqrt(1000*Air_Base.Constants.MM*T)/(0.36^2*Omega);
  //Ideal-gas part of thermal conductivity
  lambda_0 := Ncon[1]*eta_0 + Ncon[2]*f.tau^tcon[2] + Ncon[3]*f.tau^tcon[3];
  //Residual part of thermal conductivity
  for i in 4:9 loop
    lambda_r := lambda_r + Ncon[i]*f.tau^tcon[i]*f.delta^dcon[i]*exp(-gammacon[
      i]*f.delta^lcon[i]);
  end for;
  //Derivative of p w.r.t. d at constant temperature
  pddT := Air_Base.Constants.R*T*(1 + 2*f.delta*(f.fdelta - 1/f.delta) + f.delta
    ^2*(f.fdeltadelta + 1/f.delta^2));
  //chi_tilde at the given state
  xi := Air_Base.Constants.pred*(d/Air_Base.Constants.MM)/Air_Base.Constants.rhored
    ^2/(pddT*Air_Base.Constants.MM);
  //Thermal conductivity critical enhancement
  xi := xi - xiref*265.262/T;
  if (xi <= 0) then
    lambda_c := 0;
  else
    xi := 0.11*(xi/0.055)^(0.63/1.2415);
    //Derivative of p w.r.t. T at constant p
    pdTp := Air_Base.Constants.R*d*(1 + f.delta*(f.fdelta - 1/f.delta) - f.delta
      *f.tau*f.fdeltatau);
    //Specific isochoric heat capacity
    cv := Air_Base.Constants.R*(-f.tau*f.tau*f.ftautau);
    //Specific isobaric heat capacity
    cp := cv + T*pdTp*pdTp/(d*d*pddT);
    Omega_tilde := 2/Modelica.Constants.pi*((cp - cv)/cp*atan(xi/0.31) + cv/cp*
      xi/0.31);
    Omega_0_tilde := 2/Modelica.Constants.pi*(1 - exp(-1/((0.31/xi) + 1/3*(xi/
      0.31)^2*(Air_Base.Constants.rhored/(d/Air_Base.Constants.MM))^2)));
    lambda_c := d*cp*1.380658E-023*1.01*T/(6*Modelica.Constants.pi*xi*eta_dT(d,
      T))*(Omega_tilde - Omega_0_tilde)*1E012;
  end if;
  lambda := (lambda_0 + lambda_r + lambda_c)/1000;
end lambda_dT;
