within MoMoLib.Common.Numerical.RootFinding;
function BrentMethod
  "Root-finding using the algorithm of van Wijngaarden-Dekker-Brent"
  extends Modelica.Icons.Function;
  import Modelica.Utilities.Streams.error;

  input Interfaces.InputFunction f "Function for which a root should be found";
  input Real x_min "Lower bound of search interval";
  input Real x_max "Upper bound of search interval";
  input Real tol "Tolerance of solution";

  output Real u "Approximate root of f, i.e. f(u)=0";

protected
  final constant Real EPS=Modelica.Constants.eps
    "32bit floating point precision";
  Real a=x_min;
  Real b=x_max;
  Real c=x_max;
  Real d;
  Real e;
  Real fa;
  Real fb;
  Real fc;
  Real p;
  Real q;
  Real r;
  Real s;
  Real tol1;
  Real xm;
  Real min1;
  Real min2;
  Boolean flag=false;

algorithm
  fa := f(a);
  fb := f(b);

  //Check if there is a root between x_min and x_max
  if ((fa > 0.0 and fb > 0.0) or (fa < 0.0 and fb < 0.0)) then
    error("The arguments x_min and x_max to BrentMethod(..)\n" +
      "do not bracket the root of the function f:\n" + "  x_min  = " + String(
      x_min) + "\n" + "  x_max  = " + String(x_max) + "\n" + "  f(x_min) = " +
      String(fa) + "\n" + "  f(x_max) = " + String(fb) + "\n" +
      "f(x_min) and f(x_max) must have opposite sign!");
  end if;

  fc := fb;

  while not flag loop

    //Rename a, b and c and adjust bounding interval
    if ((fb > 0.0 and fc > 0.0) or (fb < 0.0 and fc < 0.0)) then
      c := a;
      fc := fa;
      d := b - a;
      e := d;
    end if;
    if (abs(fc) < abs(fb)) then
      a := b;
      b := c;
      c := a;
      fa := fb;
      fb := fc;
      fc := fa;
    end if;

    //Check if the interval is small enough, i.e. a root is found
    tol1 := 2.0*EPS*abs(b) + 0.5*tol;
    xm := 0.5*(c - b);
    if ((abs(xm) <= tol1) or (fb == 0.0)) then
      //A root is found
      flag := true;
      u := b;
    else

      if ((abs(e) >= tol1) and (abs(fa) > abs(fb))) then
        //Attempt inverse quadratic interpolation
        s := fb/fa;
        if (a == c) then
          p := 2.0*xm*s;
          q := 1.0 - s;
        else
          q := fa/fc;
          r := fb/fc;
          p := s*(2.0*xm*q*(q - r) - (b - a)*(r - 1.0));
          q := (q - 1.0)*(r - 1.0)*(s - 1.0);
        end if;

        if (p > 0.0) then
          //Check wether in bounds
          q := -q;
        end if;

        p := abs(p);
        min1 := 3.0*xm*q - abs(tol1*q);
        min2 := abs(e*q);
        min1 := if (min1 < min2) then min1 else min2;
        if (2.0*p < min1) then
          //Accept interpolation
          e := d;
          d := p/q;
        else
          //Interpolation failed, use bisection
          d := xm;
          e := d;
        end if;
      else
        //Bounds decreasing too slowly, use bisection
        d := xm;
        e := d;
      end if;

      a := b;
      fa := fb;
      if (abs(d) > tol1) then
        b := b + d;
      else
        if (xm >= 0.0) then
          if (tol1 < 0.0) then
            tol1 := abs(tol1);
          end if;
        else
          if (tol1 >= 0.0) then
            tol1 := -tol1;
          end if;
        end if;
      end if;
      b := b + tol1;
      fb := f(b);

    end if;

  end while;

end BrentMethod;
