within MoMoLib.Common.Numerical.RootFinding;
function NewtonMethod "Root-finding using the algorithm of Newton-Raphson"
  extends Modelica.Icons.Function;
  import Modelica.Utilities.Streams.error;

  input Interfaces.InputFunction f "Function for which a root should be found";
  input Interfaces.InputFunction df "First derivative of the function f";
  input Real x_min "Lower bound of search interval";
  input Real x_max "Upper bound of search interval";
  input Real tol "Tolerance of solution";

  output Real u "Approximate root of f, i.e. f(u)=0";

protected
  final constant Real EPS=Modelica.Constants.eps
    "32bit floating point precision";
  Real dx;
  Real dxold;
  Real fv;
  Real dfv;
  Real fl;
  Real fh;
  Real temp;
  Real xl;
  Real xh;
  Boolean flag=false;

algorithm
  fl := f(x_min);
  fh := f(x_max);

  //Check if there is a root between x_min and x_max
  if ((fl > 0.0 and fh > 0.0) or (fl < 0.0 and fh < 0.0)) then
    error("The arguments x_min and x_max to NewtonMethod(..)\n" +
      "do not bracket the root of the function f:\n" + "  x_min  = " + String(
      x_min) + "\n" + "  x_max  = " + String(x_max) + "\n" + "  f(x_min) = " +
      String(fl) + "\n" + "  f(x_max) = " + String(fh) + "\n" +
      "f(x_min) and f(x_max) must have opposite sign!");
  end if;

  //Check if a root has already been found
  if (fl == 0) then
    u := x_min;
    return;
  end if;
  if (fh == 0) then
    u := x_max;
    return;
  end if;

  //Adjust x_min and x_max so that f(x_min) < 0
  if (fl < 0.0) then
    xl := x_min;
    xh := x_max;
  else
    xl := x_max;
    xh := x_min;
  end if;

  //Initialization
  u := 0.5*(xl + xh);
  dxold := abs(xh - xl);
  dx := dxold;
  fv := f(u);
  dfv := df(u);

  //Iteration
  while not flag loop

    //Bisect if Newton out of range or not decreasing fast enough
    if ((((u - xh)*dfv - fv)*((u - xl)*dfv - fv) > 0.0) or (abs(2.0*fv) > abs(
        dxold*dfv))) then
      dxold := dx;
      dx := 0.5*(xh - xl);
      u := xl + dx;
      if (xl == u) then
        return;
        //Change in root negligible
      end if;
      //Newton step ok
    else
      dxold := dx;
      dx := fv/dfv;
      temp := u;
      u := u - dx;
      if (temp == u) then
        return;
        //Change in root negligible
      end if;
    end if;

    //Convergence criterion
    if (abs(dx) < tol) then
      return;
    end if;

    //New function evaluation
    fv := f(u);
    dfv := df(u);

    //Maintain the brackets on the root
    if (fv < 0.0) then
      xl := u;
    else
      xh := u;
    end if;

  end while;

end NewtonMethod;
