within MoMoLib.MoistAir.Utilities.Water;
package IAPWS09 "Water: Ice model as described by IAPWS-09 (0 ... 273.16 K)"
  extends Modelica.Icons.MaterialPropertiesPackage;
  import SI = Modelica.SIunits;

  constant Modelica.Media.Interfaces.PartialTwoPhaseMedium.FluidConstants
    waterConstants(
    each chemicalFormula="H2O",
    each structureFormula="H2O",
    each casRegistryNumber="7732-18-5",
    each iupacName="oxidane",
    each molarMass=0.018015268,
    each criticalTemperature=647.096,
    each criticalPressure=22064.0e3,
    each criticalMolarVolume=1/322.0*0.018015268,
    each normalBoilingPoint=373.124,
    each meltingPoint=273.15,
    each triplePointTemperature=273.16,
    each triplePointPressure=611.657,
    each acentricFactor=0.344,
    each dipoleMoment=1.8,
    each hasCriticalData=true);

protected
  type MolarHeatCapacity = SI.MolarHeatCapacity (
      min=0,
      max=3.e5,
      nominal=3.e1,
      start=3.e1)
    "Type for molar heat capacity with medium specific attributes";

  type MolarDensity = Real (
      final quantity="MolarDensity",
      final unit="mol/m3",
      min=0);

  type IsothermalExpansionCoefficient = Real (
      min=0,
      max=1e8,
      unit="1");

public
  package Ice09_ph
    "Water: Ice model as described by IAPWS-09 (0 ... 273.16 K) explicit in p and h"
    extends MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base(
      ThermoStates=Choices.IndependentVariables.ph,
      final ph_explicit=true,
      final dT_explicit=false,
      final pT_explicit=false);
  end Ice09_ph;

  package Ice09_dT
    "Water: Ice model as described by IAPWS-09 (0 ... 273.16 K) explicit in d and T"
    extends MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base(
      ThermoStates=Choices.IndependentVariables.dTX,
      final ph_explicit=false,
      final dT_explicit=true,
      final pT_explicit=false);
  end Ice09_dT;

  package Ice09_pT
    "Water: Ice model as described by IAPWS-09 (0 ... 273.16 K) explicit in p and T"
    extends MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base(
      ThermoStates=Choices.IndependentVariables.pT,
      final ph_explicit=false,
      final dT_explicit=false,
      final pT_explicit=true);
  end Ice09_pT;

  partial package Ice09_Base
    "Properties of ice calculated using the equation of state as given by IAPWS-09"

    extends Modelica.Media.Interfaces.PartialPureSubstance(
      mediumName="Water",
      substanceNames={"water95"},
      singleState=false,
      SpecificEnthalpy(start=1.0e5, nominal=5.0e5),
      Density(start=1.0, nominal=1.2),
      AbsolutePressure(
        start=1e5,
        nominal=1e5,
        min=1.0,
        max=1000e6),
      Temperature(
        start=273.15,
        nominal=293.15,
        min=251.165,
        max=1273));

    constant Boolean ph_explicit
      "true if explicit in pressure and specific enthalpy";
    constant Boolean dT_explicit "true if explicit in density and temperature";
    constant Boolean pT_explicit "true if explicit in pressure and temperature";

    record Constants "Constants used in the model"
      final constant MolarHeatCapacity R_bar=8.314472;
      final constant SpecificHeatCapacity R=461.52364;
      final constant MolarMass MM=18.015268E-003;
      final constant Temperature Tred=273.16;
      final constant AbsolutePressure pred=611.657;
      final constant AbsolutePressure p0=101325;
      constant SpecificEnthalpy h_off=0;
      constant SpecificEntropy s_off=0;
    end Constants;

    redeclare record extends ThermodynamicState "thermodynamic state"
      SpecificEnthalpy h "specific enthalpy";
      Density d "density";
      Temperature T "temperature";
      AbsolutePressure p "pressure";
    end ThermodynamicState;

    redeclare replaceable model extends BaseProperties(
      h(stateSelect=if ph_explicit and preferredMediumStates then StateSelect.prefer
             else StateSelect.default),
      d(stateSelect=if dT_explicit and preferredMediumStates then StateSelect.prefer
             else StateSelect.default),
      T(stateSelect=if (pT_explicit or dT_explicit) and preferredMediumStates
             then StateSelect.prefer else StateSelect.default),
      p(stateSelect=if (pT_explicit or ph_explicit) and preferredMediumStates
             then StateSelect.prefer else StateSelect.default))
      "Base properties of water"

    equation
      MM = Constants.MM;
      if dT_explicit then
        p = pressure_dT(d, T);
        h = specificEnthalpy_dT(d, T);
      elseif ph_explicit then
        d = density_ph(p, h);
        T = temperature_ph(p, h);
      else
        h = specificEnthalpy_pT(p, T);
        d = density_pT(p, T);
      end if;
      u = h - p/d;
      R = Constants.R;
      h = state.h;
      p = state.p;
      T = state.T;
      d = state.d;
    end BaseProperties;

    redeclare function density_ph
      "Computes density as a function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input AbsolutePressure p "Pressure";
      input SpecificEnthalpy h "Specific enthalpy";
      output Density d "Density";
    algorithm
      d := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.rho_ph(p, h);
    end density_ph;

    redeclare function temperature_ph
      "Computes temperature as a function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input AbsolutePressure p "Pressure";
      input SpecificEnthalpy h "Specific enthalpy";
      output Temperature T "Temperature";
    algorithm
      T := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.T_ph(p, h);
    end temperature_ph;

    redeclare function temperature_ps
      "Compute temperature from pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input AbsolutePressure p "Pressure";
      input SpecificEntropy s "Specific entropy";
      output Temperature T "Temperature";
    algorithm
      T := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.T_ps(p, s);
    end temperature_ps;

    redeclare function density_ps
      "Computes density as a function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input AbsolutePressure p "Pressure";
      input SpecificEntropy s "Specific entropy";
      output Density d "density";
    algorithm
      d := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.rho_ps(p, s);
    end density_ps;

    redeclare function pressure_dT
      "Computes pressure as a function of density and temperature"
      extends Modelica.Icons.Function;
      input Density d "Density";
      input Temperature T "Temperature";
      output AbsolutePressure p "Pressure";
    algorithm
      p := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.p_dT(d, T);
    end pressure_dT;

    redeclare function specificEnthalpy_dT
      "Computes specific enthalpy as a function of density and temperature"
      extends Modelica.Icons.Function;
      input Density d "Density";
      input Temperature T "Temperature";
      output SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.h_dT(d, T);
    end specificEnthalpy_dT;

    redeclare function specificEnthalpy_pT
      "Computes specific enthalpy as a function of pressure and temperature"
      extends Modelica.Icons.Function;
      input AbsolutePressure p "Pressure";
      input Temperature T "Temperature";
      output SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.h_pT(p, T);
    end specificEnthalpy_pT;

    redeclare function specificEnthalpy_ps
      "Computes specific enthalpy as a function of pressure and temperature"
      extends Modelica.Icons.Function;
      input AbsolutePressure p "Pressure";
      input SpecificEntropy s "Specific entropy";
      output SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.h_ps(p, s);
    end specificEnthalpy_ps;

    redeclare function density_pT
      "Computes density as a function of pressure and temperature"
      extends Modelica.Icons.Function;
      input AbsolutePressure p "Pressure";
      input Temperature T "Temperature";
      output Density d "Density";
    algorithm
      d := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.rho_pT(p, T);
    end density_pT;

    redeclare function extends dynamicViscosity
      "Return dynamic viscosity as a function of the thermodynamic state record"
    algorithm
      eta :=
        MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.dynamicViscosity(
        state);
    end dynamicViscosity;

    redeclare function extends thermalConductivity
      "Thermal conductivity of water"
    algorithm
      lambda :=
        MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.thermalConductivity(
        state);
    end thermalConductivity;

    redeclare function extends pressure "return pressure of ideal gas"
    algorithm
      p := state.p;
    end pressure;

    redeclare function extends temperature "return temperature of ideal gas"
    algorithm
      T := state.T;
    end temperature;

    redeclare function extends density "return density of ideal gas"
    algorithm
      d := state.d;
    end density;

    redeclare function extends specificEnthalpy "Return specific enthalpy"
    algorithm
      h := state.h;
    end specificEnthalpy;

    redeclare function extends specificInternalEnergy
      "Return specific internal energy"
    algorithm
      u := state.h - state.p/state.d;
    end specificInternalEnergy;

    redeclare function extends specificGibbsEnergy
      "Return specific Gibbs energy"
    algorithm
      g := state.h - state.T*specificEntropy(state);
    end specificGibbsEnergy;

    redeclare function extends specificHelmholtzEnergy
      "Return specific Helmholtz energy"
    algorithm
      f := state.h - state.p/state.d - state.T*specificEntropy(state);
    end specificHelmholtzEnergy;

    redeclare function extends specificEntropy "specific entropy of water"
    algorithm
      if dT_explicit then
        s := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.s_dT(
          state.d, state.T);
      elseif pT_explicit then
        s := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.s_pT(
          state.p, state.T);
      else
        s := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.s_ph(
          state.p, state.h);
      end if;
    end specificEntropy;

    redeclare function extends specificHeatCapacityCp
      "specific heat capacity at constant pressure of water"

    algorithm
      if dT_explicit then
        cp := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.cp_dT(
          state.d, state.T);
      elseif pT_explicit then
        cp := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.cp_pT(
          state.p, state.T);
      else
        cp := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.cp_ph(
          state.p, state.h);
      end if;
    end specificHeatCapacityCp;

    redeclare function extends specificHeatCapacityCv
      "specific heat capacity at constant volume of water"
    algorithm
      if dT_explicit then
        cv := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.cv_dT(
          state.d, state.T);
      elseif pT_explicit then
        cv := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.cv_pT(
          state.p, state.T);
      else
        cv := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.cv_ph(
          state.p, state.h);
      end if;
    end specificHeatCapacityCv;

    redeclare function extends isentropicExponent "Return isentropic exponent"
    algorithm
      if dT_explicit then
        gamma :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.isentropicExponent_dT(
          state.d, state.T);
      elseif pT_explicit then
        gamma :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.isentropicExponent_pT(
          state.p, state.T);
      else
        gamma :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.isentropicExponent_ph(
          state.p, state.h);
      end if;
    end isentropicExponent;

    redeclare function extends isothermalCompressibility
      "Isothermal compressibility of water"
    algorithm
      if dT_explicit then
        kappa :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.kappa_dT(
          state.d, state.T);
      elseif pT_explicit then
        kappa :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.kappa_pT(
          state.p, state.T);
      else
        kappa :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.kappa_ph(
          state.p, state.h);
      end if;
    end isothermalCompressibility;

    redeclare function extends isobaricExpansionCoefficient
      "isobaric expansion coefficient of water"
    algorithm
      if dT_explicit then
        beta :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.beta_dT(
          state.d, state.T);
      elseif pT_explicit then
        beta :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.beta_pT(
          state.p, state.T);
      else
        beta :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.beta_ph(
          state.p, state.h);
      end if;
    end isobaricExpansionCoefficient;

    redeclare function extends velocityOfSound
      "Return velocity of sound as a function of the thermodynamic state record"
    algorithm
      if dT_explicit then
        a :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.velocityOfSound_dT(
          state.d, state.T);
      elseif pT_explicit then
        a :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.velocityOfSound_pT(
          state.p, state.T);
      else
        a :=
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.velocityOfSound_ph(
          state.p, state.h);
      end if;
    end velocityOfSound;

    redeclare function extends density_derh_p
      "density derivative by specific enthalpy"
    algorithm
      ddhp := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ddhp(
        state.p, state.h);
    end density_derh_p;

    redeclare function extends density_derp_h "density derivative by pressure"
    algorithm
      ddph := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ddph(
        state.p, state.h);
    end density_derp_h;

  //   redeclare function extends density_derT_p
  //     "density derivative by temperature"
  //   algorithm
  //     ddTp := IF97_Utilities.ddTp(state.p, state.h, state.phase);
  //   end density_derT_p;
  //
  //   redeclare function extends density_derp_T
  //     "density derivative by pressure"
  //   algorithm
  //     ddpT := IF97_Utilities.ddpT(state.p, state.h, state.phase);
  //   end density_derp_T;

    redeclare function extends setState_dTX
      "Return thermodynamic state of water as function of d and T"
    algorithm
      state := ThermodynamicState(
            d=d,
            T=T,
            h=specificEnthalpy_dT(d, T),
            p=pressure_dT(d, T));
    end setState_dTX;

    redeclare function extends setState_phX
      "Return thermodynamic state of water as function of p and h"
    algorithm
      state := ThermodynamicState(
            d=density_ph(p, h),
            T=temperature_ph(p, h),
            h=h,
            p=p);
    end setState_phX;

    redeclare function extends setState_psX
      "Return thermodynamic state of water as function of p and s"
    algorithm
      state := ThermodynamicState(
            d=density_ps(p, s),
            T=temperature_ps(p, s),
            h=specificEnthalpy_ps(p, s),
            p=p);
    end setState_psX;

    redeclare function extends setState_pTX
      "Return thermodynamic state of water as function of p and T"
    algorithm
      state := ThermodynamicState(
            d=density_pT(p, T),
            T=T,
            h=specificEnthalpy_pT(p, T),
            p=p);
    end setState_pTX;

    redeclare function extends setSmoothState
      "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
      import Modelica.Media.Common.smoothStep;
    algorithm
      state := ThermodynamicState(
            p=smoothStep(
              x,
              state_a.p,
              state_b.p,
              x_small),
            h=smoothStep(
              x,
              state_a.h,
              state_b.h,
              x_small),
            d=density_ph(smoothStep(
              x,
              state_a.p,
              state_b.p,
              x_small), smoothStep(
              x,
              state_a.h,
              state_b.h,
              x_small)),
            T=temperature_ph(smoothStep(
              x,
              state_a.p,
              state_b.p,
              x_small), smoothStep(
              x,
              state_a.h,
              state_b.h,
              x_small)));
    end setSmoothState;

    function sublimationPressure "Compute the sublimation pressure"
      extends Modelica.Icons.Function;
      input Temperature T "Temperature";
      output AbsolutePressure psub "Sublimation pressure";

    algorithm
      psub :=
        MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.Basic.psub(T);

    end sublimationPressure;

    function meltingPressure "Compute the melting pressure"
      extends Modelica.Icons.Function;

      input Temperature T "Temperature";
      output AbsolutePressure pmelt "Melting pressure";

    algorithm
      pmelt :=
        MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.Basic.pmelt(T);

    end meltingPressure;
    annotation (Icon(coordinateSystem(preserveAspectRatio=false, extent={{-100,
              -100},{100,100}}), graphics={Text(
              extent={{-94,84},{94,40}},
              lineColor={127,191,255},
              textString="IF97"),Text(
              extent={{-94,20},{94,-24}},
              lineColor={127,191,255},
              textString="water")}), Documentation(info="<HTML>
<p>
This model calculates medium properties
for water in the <b>liquid</b>, <b>gas</b> and <b>two phase</b> regions
according to the IAPWS/IF97 standard, i.e., the accepted industrial standard
and best compromise between accuracy and computation time.
For more details see <a href=\"modelica://Modelica.Media.Water.IF97_Utilities\">
Modelica.Media.Water.IF97_Utilities</a>. Three variable pairs can be the
independent variables of the model:
</p>
<ol>
<li>Pressure <b>p</b> and specific enthalpy <b>h</b> are the most natural choice for general applications. This is the recommended choice for most general purpose applications, in particular for power plants.</li>
<li>Pressure <b>p</b> and temperature <b>T</b> are the most natural choice for applications where water is always in the same phase, both for liquid water and steam.</li>
<li>Density <b>d</b> and temperature <b>T</b> are explicit variables of the Helmholtz function in the near-critical region and can be the best choice for applications with super-critical or near-critial states.</li>
</ol>
<p>
The following quantities are always computed:
</p>
<table border=1 cellspacing=0 cellpadding=2>
  <tr><td valign=\"top\"><b>Variable</b></td>
      <td valign=\"top\"><b>Unit</b></td>
      <td valign=\"top\"><b>Description</b></td></tr>
  <tr><td valign=\"top\">T</td>
      <td valign=\"top\">K</td>
      <td valign=\"top\">temperature</td></tr>
  <tr><td valign=\"top\">u</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific internal energy</td></tr>
  <tr><td valign=\"top\">d</td>
      <td valign=\"top\">kg/m^3</td>
      <td valign=\"top\">density</td></tr>
  <tr><td valign=\"top\">p</td>
      <td valign=\"top\">Pa</td>
      <td valign=\"top\">pressure</td></tr>
  <tr><td valign=\"top\">h</td>
      <td valign=\"top\">J/kg</td>
      <td valign=\"top\">specific enthalpy</td></tr>
</table>
<p>
In some cases additional medium properties are needed.
A component that needs these optional properties has to call
one of the functions listed in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.OptionalProperties\">
Modelica.Media.UsersGuide.MediumUsage.OptionalProperties</a> and in
<a href=\"modelica://Modelica.Media.UsersGuide.MediumUsage.TwoPhase\">
Modelica.Media.UsersGuide.MediumUsage.TwoPhase</a>.
</p>
<p>Many further properties can be computed. Using the well-known Bridgman's Tables, all first partial derivatives of the standard thermodynamic variables can be computed easily.</p>
</html>"));
  end Ice09_Base;

  package Ice09_Utilities
    "Low level and utility computation for high accuracy ice properties"
    extends Modelica.Icons.Package;

    record iter = Inverses.accuracy;
    package Basic "Fundamental equation of state"
      extends Modelica.Icons.BasesPackage;

      function Gibbs "Gibbs equation of state"
        extends Modelica.Icons.Function;
        input SI.AbsolutePressure p "Absolute pressure";
        input SI.Temperature T "Temperature";
        output MoMoLib.Common.GibbsDerivs2 g
          "Gibbs function and dervatives w.r.t. T and p";

        import Modelica.ComplexMath;

      protected
        final constant Real[5] g_0={-0.632020233449497E+006,0.655022213658955,-0.189369929326131E-007,
            0.339746123271053E-014,-0.556464869058991E-021};
        final constant Real s_0=-0.332733756492168E+004;
        final constant Complex[2] t={Complex(0.368017112855051E-001,
            0.510878114959572E-001),Complex(0.337315741065416,
            0.335449415919309)};
        final constant Complex r_1=Complex(0.447050716285388E+002,
            0.656876847463481E+002);
        final constant Complex[3] r_2={Complex(-0.725974574329220E+002, -0.781008427112870E+002),
            Complex(-0.557107698030123E-004, 0.464578634580806E-004),Complex(
            0.234801409215913E-010, -0.285651142904972E-010)};
        final constant Real pi0=Ice09_Base.Constants.p0/Ice09_Base.Constants.pred;

        //temporary variables needed for calculation
        Real g0=0.0;
        Real g0p=0.0;
        Real g0pp=0.0;
        Complex r2=Complex(0.0, 0.0);
        Complex r2p=Complex(0.0, 0.0);
        Complex r2pp=Complex(0.0, 0.0);

      algorithm
        g.p := p;
        g.T := T;
        g.R := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.R;

        //Reduced pressure
        g.pi := g.p/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred;
        //Reduced temperature
        g.theta := g.T/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.Tred;

        //calculate temporary values
        //0^0 may result in an error, so we have to use a workaround
        g0 := g_0[1];
        for k in 2:5 loop
          g0 := g0 + g_0[k]*(g.pi - pi0)^(k - 1);
        end for;
        r2 := r_2[1];
        for k in 2:3 loop
          r2 := r2 + r_2[k]*(g.pi - pi0)^(k - 1);
        end for;

        //First derivative of g0 w.r.t. pi
        g0p := g_0[2]/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred;
        for k in 3:5 loop
          g0p := g0p + g_0[k]*(k - 1)/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred
            *(g.pi - pi0)^(k - 2);
        end for;
        //First derivative of r2 w.r.t. pi
        r2p := r_2[2]/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred
           + r_2[3]*2/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred
          *(g.pi - pi0);
        //Second derivative of g0 w.r.t. pi
        g0pp := g_0[3]*2/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred
          ^2;
        for k in 4:5 loop
          g0pp := g0pp + g_0[k]*(k - 1)*(k - 2)/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred
            ^2*(g.pi - pi0)^(k - 3);
        end for;
        //Second derivative of g0 w.r.t. pi
        r2pp := r_2[3]*2/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred
          ^2;

        //Gibbs equation
        g.g := g0 - s_0*MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.Tred
          *g.theta + MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.Tred
          *ComplexMath.real(r_1*((t[1] - g.theta)*ComplexMath.log(t[1] - g.theta)
           + (t[1] + g.theta)*ComplexMath.log(t[1] + g.theta) - 2*t[1]*
          ComplexMath.log(t[1]) - g.theta^2/t[1]) + r2*((t[2] - g.theta)*
          ComplexMath.log(t[2] - g.theta) + (t[2] + g.theta)*ComplexMath.log(t[
          2] + g.theta) - 2*t[2]*ComplexMath.log(t[2]) - g.theta^2/t[2]));

        //First derivative of g w.r.t. p
        g.gp := g0p + MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.Tred
          *ComplexMath.real(r2p*((t[2] - g.theta)*ComplexMath.log(t[2] - g.theta)
           + (t[2] + g.theta)*ComplexMath.log(t[2] + g.theta) - 2*t[2]*
          ComplexMath.log(t[2]) - g.theta^2/t[2]));

        //Second derivative of g w.r.t. p
        g.gpp := g0pp + MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.Tred
          *ComplexMath.real(r2pp*((t[2] - g.theta)*ComplexMath.log(t[2] - g.theta)
           + (t[2] + g.theta)*ComplexMath.log(t[2] + g.theta) - 2*t[2]*
          ComplexMath.log(t[2]) - g.theta^2/t[2]));

        //First derivative of g w.r.t. T
        g.gT := -s_0 + ComplexMath.real(r_1*(-ComplexMath.log(t[1] - g.theta)
           + ComplexMath.log(t[1] + g.theta) - 2*g.theta/t[1]) + r2*(-
          ComplexMath.log(t[2] - g.theta) + ComplexMath.log(t[2] + g.theta) - 2
          *g.theta/t[2]));

        //Second derivative of g w.r.t. T
        g.gTT := 1/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.Tred
          *ComplexMath.real(r_1*(1/(t[1] - g.theta) + 1/(t[1] + g.theta) - 2/t[
          1]) + r2*(1/(t[2] - g.theta) + 1/(t[2] + g.theta) - 2/t[2]));

        //Mixed derivative of g w.r.t. T and p
        g.gTp := ComplexMath.real(r2p*(-ComplexMath.log(t[2] - g.theta) +
          ComplexMath.log(t[2] + g.theta) - 2*g.theta/t[2]));

      end Gibbs;

      function psub "sublimation pressure"
        extends Modelica.Icons.Function;

        input SI.Temperature T "Temperature";
        output SI.AbsolutePressure p_sub "Pressure";

      protected
        final constant Real[3] a={-0.212144006E+002,0.273203819E+002,-0.610598130E+001};
        final constant Real[3] b={0.333333333E-002,0.120666667E+001,
            0.170333333E+001};
        Real theta=0;
        Real sum=0;

      algorithm
        /*assert(T >= 50 and T < 273.16, "IAPWS-95 medium function psub: input temperature T = "
     + String(T) + " K\n" + "must be in the range 50 <= T < 273.16 K.");*/
        theta := T/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.Tred;

        for k in 1:3 loop
          sum := sum + a[k]*theta^b[k];
        end for;

        p_sub := exp(sum/theta)*MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred;

      end psub;

      function pmelt "melting pressure"
        extends Modelica.Icons.Function;

        input SI.Temperature T "Temperature";
        output SI.AbsolutePressure pmelt "Pressure";

      protected
        final constant Real[3] a={0.119539337E+007,0.808183159E+005,
            0.333826860E+004};
        final constant Real[3] b={0.3E+001,0.2575E+002,0.103750E+003};
        Real theta=0;
        Real sum=0;

      algorithm
        theta := T/MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.Tred;

        for k in 1:3 loop
          sum := sum + a[k]*(1 - theta^b[k]);
        end for;

        pmelt := (1 + sum)*MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.pred;

      end pmelt;

      function Tsub "sublimation temperature"
        extends Modelica.Icons.Function;
        input Modelica.SIunits.AbsolutePressure p "Pressure";
        output Modelica.SIunits.Temperature Tsub "Temperature";

      protected
        function Tsub_res
          extends MoMoLib.Common.Numerical.Interfaces.InputFunction;
          input Modelica.SIunits.AbsolutePressure p "Pressure";

        algorithm
          y :=
            MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.Basic.psub(
            x) - p;

        end Tsub_res;

      algorithm
        Tsub := MoMoLib.Common.Numerical.RootFinding.BrentMethod(
                function Tsub_res(p=p),
                143.15,
                2000,
                1e-9);

      end Tsub;
    end Basic;

    package Inverses "Inverse function"
      extends Modelica.Icons.BasesPackage;

      record accuracy "Accuracy of the iterations"
        extends Modelica.Icons.Record;
        constant Real deld=1E-006 "Accuracy of p";
        constant Real delh=1E-006 "Accuracy of h";
        constant Real dels=1E-006 "Accuracy of s";
      end accuracy;

      function Tofph "Return T as a function of p and h"
        extends Modelica.Icons.Function;
        input SI.Pressure p "pressure";
        input SI.SpecificEnthalpy h "specific enthalpy";
        input SI.SpecificEnthalpy delh "iteration accuracy";
        output SI.Temperature T "temperature (K)";

      protected
        Integer i=0 "loop counter";
        Real dh "enthalpy difference";
        Real delT "temperature step";
        MoMoLib.Common.GibbsDerivs2 g
          "dimensionless Gibbs function and dervatives";
        Modelica.Media.Common.NewtonDerivatives_ph nDerivs
          "derivatives needed in Newton iteration";
        Boolean found=false "flag for iteration success";

      algorithm
        T := 253.15;

        while ((i < 1000) and not found) loop
          g := Basic.Gibbs(p, T);
          nDerivs := MoMoLib.Common.Gibbs2_ph(g);
          dh := nDerivs.h - MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.h_off
             - h;
          if (abs(dh) <= delh) then
            found := true;
          end if;
          delT := dh/nDerivs.ht;
          T := T - delT;
          i := i + 1;
        end while;
      end Tofph;

      function pofdT "Return p as a function of d and T"
        extends Modelica.Icons.Function;
        input SI.Density d "density";
        input SI.Temperature T "temperature";
        input SI.Density deld "iteration accuracy";
        output SI.Pressure p "pressure";

      protected
        Integer i=0 "loop counter";
        SI.SpecificVolume v "specific volume";
        SI.SpecificVolume dv "specific volume difference";
        SI.Density dd "density difference";
        Real delp "pressure step";
        MoMoLib.Common.GibbsDerivs2 g
          "dimensionless Gibbs function and dervatives";
        MoMoLib.Common.NewtonDerivatives_dT nDerivs
          "derivatives needed in Newton iteration";
        Boolean found=false "flag for iteration success";

      algorithm
        p := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.R*T*
          d;
        v := 1/d;

        while ((i < 1000) and not found) loop
          g := Basic.Gibbs(p, T);
          nDerivs := MoMoLib.Common.Gibbs2_dT(g);
          dv := nDerivs.v - v;
          dd := 1/nDerivs.v - d;
          if (abs(dd) <= deld) then
            found := true;
          end if;
          delp := dv/nDerivs.vp;
          p := p - delp;
          i := i + 1;
        end while;
      end pofdT;

      function Tofps "Return T as a function of p and s"
        extends Modelica.Icons.Function;
        input SI.Pressure p "Pressure";
        input SI.SpecificEntropy s "Specific entropy";
        input Real dels "Accuracy of s";
        output SI.Temperature T "Temperature";

      protected
        Integer i=0 "loop counter";
        Real ds "entropy difference";
        Real delT "temperature step";
        MoMoLib.Common.GibbsDerivs2 g
          "dimensionless Gibbs function and dervatives";
        Modelica.Media.Common.NewtonDerivatives_ps nDerivs
          "derivatives needed in Newton iteration";
        Boolean found=false "flag for iteration success";

      algorithm
        T := 253.15;

        while ((i < 1000) and not found) loop
          g := Basic.Gibbs(p, T);
          nDerivs := MoMoLib.Common.Gibbs2_ps(g);
          ds := nDerivs.s - MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.s_off
             - s;
          if (abs(ds) <= dels) then
            found := true;
          end if;
          delT := ds/nDerivs.st;
          T := T - delT;
          i := i + 1;
        end while;
      end Tofps;
    end Inverses;

    package Transport "Transport properties for air"
      extends Modelica.Icons.BasesPackage;

      function eta_pT "Return dynamic viscosity as a function of p and T"
        extends Modelica.Icons.Function;
        input SI.Pressure p "Pressure";
        input SI.Temperature T "Temperature";
        output SI.DynamicViscosity eta "Dynamic viscosity";

      algorithm
        eta := 0;
      end eta_pT;

      function lambda_pT "Return thermal conductivity as a function of p and T"
        extends Modelica.Icons.Function;
        input SI.Pressure p "Pressure";
        input SI.Temperature T "Temperature";
        output SI.ThermalConductivity lambda "Thermal conductivity";

      algorithm
        lambda := 0;
      end lambda_pT;
    end Transport;

    function ice09BaseProp_ps "intermediate property record for water"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEntropy s "specific entropy";
      output MoMoLib.Common.Properties aux "auxiliary record";
    protected
      MoMoLib.Common.GibbsDerivs2 g
        "Gibbs funcion and dervatives w.r.t. p and T";
    algorithm
      aux.p := p;
      aux.s := s;
      aux.R := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.R;
      aux.T := Inverses.Tofps(
            p=p,
            s=s,
            dels=iter.dels);
      g := Basic.Gibbs(aux.p, aux.T);
      aux.rho := 1/g.gp;
      aux.h := g.g - g.T*g.gT - MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.h_off;
      aux.cp := -g.T*g.gTT;
      aux.cv := aux.cp - g.T/g.gpp*g.gTp^2;
      aux.vt := 1/aux.rho*g.gTp/g.gp;
      aux.vp := 1/aux.rho*g.gpp/g.gp;
    end ice09BaseProp_ps;

    function rho_props_ps
      "density as function of pressure and specific entropy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEntropy s "specific entropy";
      input MoMoLib.Common.Properties properties "auxiliary record";
      output Modelica.SIunits.Density rho "density";
    algorithm
      rho := properties.rho;
      annotation (Inline=false, LateInline=true);
    end rho_props_ps;

    function rho_ps "density as function of pressure and specific entropy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEntropy s "specific entropy";
      output Modelica.SIunits.Density rho "density";
    algorithm
      rho := rho_props_ps(
            p,
            s,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ps(
          p, s));

    end rho_ps;

    function T_props_ps
      "temperature as function of pressure and specific entropy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEntropy s "specific entropy";
      input MoMoLib.Common.Properties properties "auxiliary record";
      output Modelica.SIunits.Temperature T "temperature";
    algorithm
      T := properties.T;
      annotation (Inline=false, LateInline=true);
    end T_props_ps;

    function T_ps "temperature as function of pressure and specific entropy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEntropy s "specific entropy";
      output Modelica.SIunits.Temperature T "Temperature";
    algorithm
      T := T_props_ps(
            p,
            s,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ps(
          p, s));

    end T_ps;

    function h_props_ps
      "specific enthalpy as function or pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEntropy s "specific entropy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := aux.h;
      annotation (Inline=false, LateInline=true);
    end h_props_ps;

    function h_ps "specific enthalpy as function or pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEntropy s "specific entropy";
      output Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := h_props_ps(
            p,
            s,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ps(
          p, s));

    end h_ps;

    function ice09BaseProp_ph "intermediate property record for water"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output MoMoLib.Common.Properties aux "auxiliary record";
    protected
      MoMoLib.Common.GibbsDerivs2 g
        "dimensionless Gibbs funcion and dervatives";
      Integer error "error flag for inverse iterations";
    algorithm
      aux.p := p;
      aux.h := h;
      aux.R := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.R;
      aux.T := Inverses.Tofph(
            p,
            h,
            delh=iter.delh);
      g := Basic.Gibbs(aux.p, aux.T);
      aux.rho := 1/g.gp;
      aux.s := -g.gT - MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.s_off;
      aux.cp := -g.T*g.gTT;
      aux.cv := aux.cp - g.T/g.gpp*g.gTp^2;
      aux.vt := 1/aux.rho*g.gTp/g.gp;
      aux.vp := 1/aux.rho*g.gpp/g.gp;
    end ice09BaseProp_ph;

    function rho_props_ph
      "density as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties properties "auxiliary record";
      output Modelica.SIunits.Density rho "density";
    algorithm
      rho := properties.rho;
      annotation (
        derivative(noDerivative=properties) = rho_ph_der,
        Inline=false,
        LateInline=true);
    end rho_props_ph;

    function rho_ph "density as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.Density rho "density";
    algorithm
      rho := rho_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end rho_ph;

    function rho_ph_der "derivative function of rho_ph"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      input Real p_der "derivative of pressure";
      input Real h_der "derivative of specific enthalpy";
      output Real rho_der "derivative of density";
    algorithm
      rho_der := ((aux.rho*(aux.cv*aux.rho + aux.pt))/(aux.rho*aux.rho*aux.pd*
        aux.cv + aux.T*aux.pt*aux.pt))*p_der + (-aux.rho*aux.rho*aux.pt/(aux.rho
        *aux.rho*aux.pd*aux.cv + aux.T*aux.pt*aux.pt))*h_der;
    end rho_ph_der;

    function T_props_ph
      "temperature as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties properties "auxiliary record";
      output Modelica.SIunits.Temperature T "temperature";
    algorithm
      T := properties.T;
      annotation (
        derivative(noDerivative=properties) = T_ph_der,
        Inline=false,
        LateInline=true);
    end T_props_ph;

    function T_ph "temperature as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.Temperature T "Temperature";
    algorithm
      T := T_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end T_ph;

    function T_ph_der "derivative function of T_ph"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      input Real p_der "derivative of pressure";
      input Real h_der "derivative of specific enthalpy";
      output Real T_der "derivative of temperature";
    algorithm
      T_der := ((-aux.rho*aux.pd + aux.T*aux.pt)/(aux.rho*aux.rho*aux.pd*aux.cv
         + aux.T*aux.pt*aux.pt))*p_der + ((aux.rho*aux.rho*aux.pd)/(aux.rho*aux.rho
        *aux.pd*aux.cv + aux.T*aux.pt*aux.pt))*h_der;
    end T_ph_der;

    function s_props_ph
      "specific entropy as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties properties "auxiliary record";
      output Modelica.SIunits.SpecificEntropy s "specific entropy";
    algorithm
      s := properties.s;
      annotation (
        derivative(noDerivative=properties) = s_ph_der,
        Inline=false,
        LateInline=true);
    end s_props_ph;

    function s_ph
      "specific entropy as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.SpecificEntropy s "specific entropy";
    algorithm
      s := s_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end s_ph;

    function s_ph_der
      "specific entropy as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      input Real p_der "derivative of pressure";
      input Real h_der "derivative of specific enthalpy";
      output Real s_der "derivative of entropy";
    algorithm
      s_der := -1/(aux.rho*aux.T)*p_der + 1/aux.T*h_der;
    end s_ph_der;

    function cv_props_ph
      "specific heat capacity at constant volume as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificHeatCapacity cv "specific heat capacity";
    algorithm
      cv := aux.cv;
      annotation (Inline=false, LateInline=true);
    end cv_props_ph;

    function cv_ph
      "specific heat capacity at constant volume as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.SpecificHeatCapacity cv "specific heat capacity";
    algorithm
      cv := cv_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end cv_ph;

    function cp_props_ph
      "specific heat capacity at constant pressure as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificHeatCapacity cp "specific heat capacity";
    algorithm
      cp := aux.cp;
      annotation (Inline=false, LateInline=true);
    end cp_props_ph;

    function cp_ph
      "specific heat capacity at constant pressure as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.SpecificHeatCapacity cp "specific heat capacity";
    algorithm
      cp := cp_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end cp_ph;

    function beta_props_ph
      "isobaric expansion coefficient as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.RelativePressureCoefficient beta
        "isobaric expansion coefficient";
    algorithm
      beta := aux.pt/(aux.rho*aux.pd);
      annotation (Inline=false, LateInline=true);
    end beta_props_ph;

    function beta_ph
      "isobaric expansion coefficient as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.RelativePressureCoefficient beta
        "isobaric expansion coefficient";
    algorithm
      beta := beta_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end beta_ph;

    function kappa_props_ph
      "isothermal compressibility factor as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.IsothermalCompressibility kappa
        "isothermal compressibility factor";
    algorithm
      kappa := 1/(aux.rho*aux.pd);
      annotation (Inline=false, LateInline=true);
    end kappa_props_ph;

    function kappa_ph
      "isothermal compressibility factor as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.IsothermalCompressibility kappa
        "isothermal compressibility factor";
    algorithm
      kappa := kappa_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end kappa_ph;

    function velocityOfSound_props_ph
      "speed of sound as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.Velocity a "speed of sound";
    algorithm
      a := sqrt(max(0, aux.pd + aux.pt*aux.pt*aux.T/(aux.rho*aux.rho*aux.cv)));
      annotation (Inline=false, LateInline=true);
    end velocityOfSound_props_ph;

    function velocityOfSound_ph
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.Velocity a "speed of sound";
    algorithm
      a := velocityOfSound_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end velocityOfSound_ph;

    function isentropicExponent_props_ph
      "isentropic exponent as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Real gamma "isentropic exponent";
    algorithm
      gamma := 1/(aux.rho*p)*((aux.pd*aux.cv*aux.rho*aux.rho + aux.pt*aux.pt*
        aux.T)/(aux.cv));
      annotation (Inline=false, LateInline=true);
    end isentropicExponent_props_ph;

    function isentropicExponent_ph
      "isentropic exponent as function of pressure and specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Real gamma "isentropic exponent";
    algorithm
      gamma := isentropicExponent_props_ph(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

      annotation (Inline=false, LateInline=true);
    end isentropicExponent_ph;

    function ddph_props "density derivative by pressure"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.DerDensityByPressure ddph
        "density derivative by pressure";
    algorithm
      ddph := ((aux.rho*(aux.cv*aux.rho + aux.pt))/(aux.rho*aux.rho*aux.pd*aux.cv
         + aux.T*aux.pt*aux.pt));
      annotation (Inline=false, LateInline=true);
    end ddph_props;

    function ddph "density derivative by pressure"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.DerDensityByPressure ddph
        "density derivative by pressure";
    algorithm
      ddph := ddph_props(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end ddph;

    function ddhp_props "density derivative by specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.DerDensityByEnthalpy ddhp
        "density derivative by specific enthalpy";
    algorithm
      ddhp := -aux.rho*aux.rho*aux.pt/(aux.rho*aux.rho*aux.pd*aux.cv + aux.T*
        aux.pt*aux.pt);
      annotation (Inline=false, LateInline=true);
    end ddhp_props;

    function ddhp "density derivative by specific enthalpy"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
      output Modelica.SIunits.DerDensityByEnthalpy ddhp
        "density derivative by specific enthalpy";
    algorithm
      ddhp := ddhp_props(
            p,
            h,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_ph(
          p, h));

    end ddhp;

    function ice09BaseProp_pT
      "intermediate property record for water (p and T prefered states)"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output MoMoLib.Common.Properties aux "auxiliary record";
    protected
      MoMoLib.Common.GibbsDerivs2 g
        "Gibbs funcion and dervatives w.r.t. p and T";
    algorithm
      aux.p := p;
      aux.T := T;
      aux.R := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.R;
      g := Basic.Gibbs(aux.p, T);
      aux.rho := 1/g.gp;
      aux.h := g.g - g.T*g.gT - MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.h_off;
      aux.s := -g.gT - MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.s_off;
      aux.cp := -g.T*g.gTT;
      aux.cv := aux.cp - g.T/g.gpp*g.gTp^2;
      aux.vt := 1/aux.rho*g.gTp/g.gp;
      aux.vp := 1/aux.rho*g.gpp/g.gp;
    end ice09BaseProp_pT;

    function rho_props_pT "density as function or pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.Density rho "density";
    algorithm
      rho := aux.rho;
      annotation (
        derivative(noDerivative=aux) = rho_pT_der,
        Inline=false,
        LateInline=true);
    end rho_props_pT;

    function rho_pT "density as function or pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.Density rho "density";
    algorithm
      rho := rho_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end rho_pT;

    function rho_pT_der "derivative function of rho_pT"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      input Real p_der "derivative of pressure";
      input Real T_der "derivative of temperature";
      output Real rho_der "derivative of density";
    algorithm
      rho_der := (1/aux.pd)*p_der - (aux.pt/aux.pd)*T_der;
    end rho_pT_der;

    function h_props_pT
      "specific enthalpy as function or pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := aux.h;
      annotation (
        derivative(noDerivative=aux) = h_pT_der,
        Inline=false,
        LateInline=true);
    end h_props_pT;

    function h_pT "specific enthalpy as function or pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "Temperature";
      output Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := h_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end h_pT;

    function h_pT_der "derivative function of h_pT"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      input Real p_der "derivative of pressure";
      input Real T_der "derivative of temperature";
      output Real h_der "derivative of specific enthalpy";
    algorithm
      h_der := ((-aux.rho*aux.pd + T*aux.pt)/(aux.rho*aux.rho*aux.pd))*p_der +
        ((aux.rho*aux.rho*aux.pd*aux.cv + aux.T*aux.pt*aux.pt)/(aux.rho*aux.rho
        *aux.pd))*T_der;
    end h_pT_der;

    function s_props_pT
      "specific entropy as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificEntropy s "specific entropy";
    algorithm
      s := aux.s;
      annotation (Inline=false, LateInline=true);
    end s_props_pT;

    function s_pT "temperature as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.SpecificEntropy s "specific entropy";
    algorithm
      s := s_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end s_pT;

    function cv_props_pT
      "specific heat capacity at constant volume as function of pressure and temperature"

      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificHeatCapacity cv "specific heat capacity";
    algorithm
      cv := aux.cv;
      annotation (Inline=false, LateInline=true);
    end cv_props_pT;

    function cv_pT
      "specific heat capacity at constant volume as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.SpecificHeatCapacity cv "specific heat capacity";
    algorithm
      cv := cv_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end cv_pT;

    function cp_props_pT
      "specific heat capacity at constant pressure as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificHeatCapacity cp "specific heat capacity";
    algorithm
      cp := aux.cv + aux.T*aux.pt*aux.pt/(aux.rho*aux.rho*aux.pd);
      annotation (Inline=false, LateInline=true);
    end cp_props_pT;

    function cp_pT
      "specific heat capacity at constant pressure as function of pressure and temperature"

      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.SpecificHeatCapacity cp "specific heat capacity";
    algorithm
      cp := cp_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end cp_pT;

    function beta_props_pT
      "isobaric expansion coefficient as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.RelativePressureCoefficient beta
        "isobaric expansion coefficient";
    algorithm
      beta := aux.pt/(aux.rho*aux.pd);
      annotation (Inline=false, LateInline=true);
    end beta_props_pT;

    function beta_pT
      "isobaric expansion coefficient as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.RelativePressureCoefficient beta
        "isobaric expansion coefficient";
    algorithm
      beta := beta_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end beta_pT;

    function kappa_props_pT
      "isothermal compressibility factor as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.IsothermalCompressibility kappa
        "isothermal compressibility factor";
    algorithm
      kappa := -aux.rho*aux.vp;
      annotation (Inline=false, LateInline=true);
    end kappa_props_pT;

    function kappa_pT
      "isothermal compressibility factor as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.IsothermalCompressibility kappa
        "isothermal compressibility factor";
    algorithm
      kappa := kappa_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end kappa_pT;

    function velocityOfSound_props_pT
      "speed of sound as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.Velocity a "speed of sound";
    algorithm
      a := sqrt(max(0, (aux.pd*aux.rho*aux.rho*aux.cv + aux.pt*aux.pt*aux.T)/(
        aux.rho*aux.rho*aux.cv)));
      annotation (Inline=false, LateInline=true);
    end velocityOfSound_props_pT;

    function velocityOfSound_pT
      "speed of sound as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.Velocity a "speed of sound";
    algorithm
      a := velocityOfSound_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end velocityOfSound_pT;

    function isentropicExponent_props_pT
      "isentropic exponent as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Real gamma "isentropic exponent";
    algorithm
      gamma := 1/(aux.rho*p)*((aux.pd*aux.cv*aux.rho*aux.rho + aux.pt*aux.pt*
        aux.T)/(aux.cv));
      annotation (Inline=false, LateInline=true);
    end isentropicExponent_props_pT;

    function isentropicExponent_pT
      "isentropic exponent as function of pressure and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Pressure p "pressure";
      input Modelica.SIunits.Temperature T "temperature";
      output Real gamma "isentropic exponent";
    algorithm
      gamma := isentropicExponent_props_pT(
            p,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_pT(
          p, T));

    end isentropicExponent_pT;

    function ice09BaseProp_dT "intermediate property record for water"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      output MoMoLib.Common.Properties aux "auxiliary record";
    protected
      MoMoLib.Common.GibbsDerivs2 g
        "dimensionless Gibbs funcion and dervatives";
    algorithm
      aux.rho := d;
      aux.T := T;
      aux.R := MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.R;
      aux.p := Inverses.pofdT(
            d,
            T,
            deld=iter.deld);
      g := Basic.Gibbs(aux.p, T);
      aux.h := g.g - g.T*g.gT - MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.h_off;
      aux.s := -g.gT - MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.Constants.s_off;
      aux.cp := -g.T*g.gTT;
      aux.cv := aux.cp - g.T/g.gpp*g.gTp^2;
      aux.vt := 1/aux.rho*g.gTp/g.gp;
      aux.vp := 1/aux.rho*g.gpp/g.gp;
    end ice09BaseProp_dT;

    function h_props_dT
      "specific enthalpy as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "Temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := aux.h;
      annotation (
        derivative(noDerivative=aux) = h_dT_der,
        Inline=false,
        LateInline=true);
    end h_props_dT;

    function h_dT "specific enthalpy as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "Temperature";
      output Modelica.SIunits.SpecificEnthalpy h "specific enthalpy";
    algorithm
      h := h_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end h_dT;

    function h_dT_der "derivative function of h_dT"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      input Real d_der "derivative of density";
      input Real T_der "derivative of temperature";
      output Real h_der "derivative of specific enthalpy";
    algorithm
      h_der := ((-d*aux.pd + T*aux.pt)/(d*d))*d_der + ((aux.cv*d + aux.pt)/d)*
        T_der;
    end h_dT_der;

    function p_props_dT "pressure as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "Temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.Pressure p "pressure";
    algorithm
      p := aux.p;
      annotation (
        derivative(noDerivative=aux) = p_dT_der,
        Inline=false,
        LateInline=true);
    end p_props_dT;

    function p_dT "pressure as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "Temperature";
      output Modelica.SIunits.Pressure p "pressure";
    algorithm
      p := p_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end p_dT;

    function p_dT_der "derivative function of p_dT"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      input Real d_der "derivative of density";
      input Real T_der "derivative of temperature";
      output Real p_der "derivative of pressure";
    algorithm
      p_der := aux.pd*d_der + aux.pt*T_der;
    end p_dT_der;

    function s_props_dT
      "specific entropy as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "Temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificEntropy s "specific entropy";
    algorithm
      s := aux.s;
      annotation (Inline=false, LateInline=true);
    end s_props_dT;

    function s_dT "temperature as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "Temperature";
      output Modelica.SIunits.SpecificEntropy s "specific entropy";
    algorithm
      s := s_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end s_dT;

    function cv_props_dT
      "specific heat capacity at constant volume as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificHeatCapacity cv "specific heat capacity";
    algorithm
      cv := aux.cv;
      annotation (Inline=false, LateInline=true);
    end cv_props_dT;

    function cv_dT
      "specific heat capacity at constant volume as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.SpecificHeatCapacity cv "specific heat capacity";
    algorithm
      cv := cv_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end cv_dT;

    function cp_props_dT
      "specific heat capacity at constant pressure as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.SpecificHeatCapacity cp "specific heat capacity";
    algorithm
      cp := aux.cp;
      annotation (Inline=false, LateInline=true);
    end cp_props_dT;

    function cp_dT
      "specific heat capacity at constant pressure as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.SpecificHeatCapacity cp "specific heat capacity";
    algorithm
      cp := cp_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end cp_dT;

    function beta_props_dT
      "isobaric expansion coefficient as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.RelativePressureCoefficient beta
        "isobaric expansion coefficient";
    algorithm
      beta := aux.pt/(aux.rho*aux.pd);
      annotation (Inline=false, LateInline=true);
    end beta_props_dT;

    function beta_dT
      "isobaric expansion coefficient as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.RelativePressureCoefficient beta
        "isobaric expansion coefficient";
    algorithm
      beta := beta_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end beta_dT;

    function kappa_props_dT
      "isothermal compressibility factor as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.IsothermalCompressibility kappa
        "isothermal compressibility factor";
    algorithm
      kappa := 1/(aux.rho*aux.pd);
      annotation (Inline=false, LateInline=true);
    end kappa_props_dT;

    function kappa_dT
      "isothermal compressibility factor as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.IsothermalCompressibility kappa
        "isothermal compressibility factor";
    algorithm
      kappa := kappa_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end kappa_dT;

    function velocityOfSound_props_dT
      "speed of sound as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Modelica.SIunits.Velocity a "speed of sound";
    algorithm
      a := sqrt(max(0, ((aux.pd*aux.rho*aux.rho*aux.cv + aux.pt*aux.pt*aux.T)/(
        aux.rho*aux.rho*aux.cv))));
      annotation (Inline=false, LateInline=true);
    end velocityOfSound_props_dT;

    function velocityOfSound_dT
      "speed of sound as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      output Modelica.SIunits.Velocity a "speed of sound";
    algorithm
      a := velocityOfSound_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end velocityOfSound_dT;

    function isentropicExponent_props_dT
      "isentropic exponent as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      input MoMoLib.Common.Properties aux "auxiliary record";
      output Real gamma "isentropic exponent";
    algorithm
      gamma := 1/(aux.rho*aux.p)*((aux.pd*aux.cv*aux.rho*aux.rho + aux.pt*aux.pt
        *aux.T)/(aux.cv));
      annotation (Inline=false, LateInline=true);
    end isentropicExponent_props_dT;

    function isentropicExponent_dT
      "isentropic exponent as function of density and temperature"
      extends Modelica.Icons.Function;
      input Modelica.SIunits.Density d "density";
      input Modelica.SIunits.Temperature T "temperature";
      output Real gamma "isentropic exponent";
    algorithm
      gamma := isentropicExponent_props_dT(
            d,
            T,
          MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.ice09BaseProp_dT(
          d, T));

    end isentropicExponent_dT;

    function dynamicViscosity
      "Return dynamic viscosity as a function of the thermodynamic state record"
      extends Modelica.Icons.Function;
      input
        MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.ThermodynamicState
        state "Thermodynamic state record";
      output SI.DynamicViscosity eta "Dynamic viscosity";
    algorithm
      eta :=
        MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.Transport.eta_pT(
        state.p, state.T);
    end dynamicViscosity;

    function thermalConductivity
      "Return thermal conductivity as a function of the thermodynamic state record"
      extends Modelica.Icons.Function;
      input
        MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Base.ThermodynamicState
        state "Thermodynamic state record";
      output SI.ThermalConductivity lambda "Thermal conductivity";
    algorithm
      lambda :=
        MoMoLib.MoistAir.Utilities.Water.IAPWS09.Ice09_Utilities.Transport.lambda_pT(
        state.p, state.T);
    end thermalConductivity;
    annotation (Documentation(info="<HTML>
      <h4>Package description:</h4>
      <p>This package provides high accuracy physical properties for water according
      to the IAPWS/IF97 standard. It has been part of the ThermoFluid Modelica library and been extended,
      reorganized and documented to become part of the Modelica Standard library.</p>
      <p>An important feature that distinguishes this implementation of the IF97 steam property standard
      is that this implementation has been explicitly designed to work well in dynamic simulations. Computational
      performance has been of high importance. This means that there often exist several ways to get the same result
      from different functions if one of the functions is called often but can be optimized for that purpose.
      </p>
      <p>
      The original documentation of the IAPWS/IF97 steam properties can freely be distributed with computer
      implementations, so for curious minds the complete standard documentation is provided with the Modelica
      properties library. The following documents are included
      (in directory Modelica/Resources/Documentation/Media/Water/IF97documentation):
      </p>
      <ul>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a> The standards document for the main part of the IF97.</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/Back3.pdf\">Back3.pdf</a> The backwards equations for region 3.</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/crits.pdf\">crits.pdf</a> The critical point data.</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/meltsub.pdf\">meltsub.pdf</a> The melting- and sublimation line formulation (in IF97_Utilities.BaseIF97.IceBoundaries)</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/surf.pdf\">surf.pdf</a> The surface tension standard definition</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/thcond.pdf\">thcond.pdf</a> The thermal conductivity standard definition</li>
      <li><a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/visc.pdf\">visc.pdf</a> The viscosity standard definition</li>
      </ul>
      <h4>Package contents
      </h4>
      <ul>
      <li>Package <b>BaseIF97</b> contains the implementation of the IAPWS-IF97 as described in
      <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/IF97.pdf\">IF97.pdf</a>. The explicit backwards equations for region 3 from
      <a href=\"modelica://Modelica/Resources/Documentation/Media/Water/IF97documentation/Back3.pdf\">Back3.pdf</a> are implemented as initial values for an inverse iteration of the exact
      function in IF97 for the input pairs (p,h) and (p,s).
      The low-level functions in BaseIF97 are not needed for standard simulation usage,
      but can be useful for experts and some special purposes.</li>
      <li>Function <b>water_ph</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and specific entropy enthalpy h as dynamic states in the record ThermoProperties_ph. </li>
      <li>Function <b>water_ps</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and specific entropy s as dynamic states in the record ThermoProperties_ps. </li>
      <li>Function <b>water_dT</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using density d and temperature T as dynamic states in the record ThermoProperties_dT. </li>
      <li>Function <b>water_pT</b> returns all properties needed for a dynamic control volume model and properties of general
      interest using pressure p and temperature T as dynamic states in the record ThermoProperties_pT. Due to the coupling of
      pressure and temperature in the two-phase region, this model can obviously
      only be used for one-phase models or models treating both phases independently.</li>
      <li>Function <b>hl_p</b> computes the liquid specific enthalpy as a function of pressure. For overcritical pressures,
      the critical specific enthalpy is returned</li>
      <li>Function <b>hv_p</b> computes the vapour specific enthalpy as a function of pressure. For overcritical pressures,
      the critical specific enthalpy is returned</li>
      <li>Function <b>sl_p</b> computes the liquid specific entropy as a function of pressure. For overcritical pressures,
      the critical  specific entropy is returned</li>
      <li>Function <b>sv_p</b> computes the vapour  specific entropy as a function of pressure. For overcritical pressures,
      the critical  specific entropyis returned</li>
      <li>Function <b>rhol_T</b> computes the liquid density as a function of temperature. For overcritical temperatures,
      the critical density is returned</li>
      <li>Function <b>rhol_T</b> computes the vapour density as a function of temperature. For overcritical temperatures,
      the critical density is returned</li>
      <li>Function <b>dynamicViscosity</b> computes the dynamic viscosity as a function of density and temperature.</li>
      <li>Function <b>thermalConductivity</b> computes the thermal conductivity as a function of density, temperature and pressure.
      <b>Important note</b>: Obviously only two of the three
      inputs are really needed, but using three inputs speeds up the computation and the three variables
      are known in most models anyways. The inputs d,T and p have to be consistent.</li>
      <li>Function <b>surfaceTension</b> computes the surface tension between vapour
          and liquid water as a function of temperature.</li>
      <li>Function <b>isentropicEnthalpy</b> computes the specific enthalpy h(p,s,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary.</li>
      <li>Function <b>dynamicIsentropicEnthalpy</b> computes the specific enthalpy h(p,s,,dguess,Tguess,phase) in all regions.
          The phase input is needed due to discontinuous derivatives at the phase boundary. Tguess and dguess are initial guess
          values for the density and temperature consistent with p and s. This function should be preferred in
          dynamic simulations where good guesses are often available.</li>
      </ul>
      <h4>Version Info and Revision history
      </h4>
      <ul>
      <li>First implemented: <i>July, 2000</i>
      by Hubertus Tummescheit for the ThermoFluid Library with help from Jonas Eborn and Falko Jens Wagner
      </li>
      <li>Code reorganization, enhanced documentation, additional functions:   <i>December, 2002</i>
      by <a href=\"mailto:Hubertus.Tummescheit@modelon.se\">Hubertus Tummescheit</a> and moved to Modelica
      properties library.</li>
      </ul>
      <address>Author: Hubertus Tummescheit, <br>
      Modelon AB<br>
      Ideon Science Park<br>
      SE-22370 Lund, Sweden<br>
      email: hubertus@modelon.se
      </address>
      </HTML>", revisions="<h4>Intermediate release notes during development</h4>
<p>Currenly the Events/noEvents switch is only implmented for p-h states. Only after testing that implmentation, it will be extended to dT.</p>"));
  end Ice09_Utilities;
  annotation (Documentation(info="<html>
<p>Calculation of fluid properties for ice in the region from 0 Kelvin to 273.16 Kelvin at pressures up to 210 MPa.
</p>

<h4><font color=\"#EF9B13\">Restriction</font></h4>
The functions provided by this package shall be used inside of the restricted limits according to the referenced literature.
</p>

<ul>
<li>
<b>0 &le; p &le; 210 MPa</b>
</li>
<li>
<b>0 K &le; T &le; 273.16 K</i>
</li>
</ul>

<h4><font style=\"color: #ef9b13; \">References</font></h4>
<dl>
<dd><b>Revised Release on the Equation of State 2006 for H2O Ice Ih</b>. 2009 International Association for the Properties of Water and Steam.
</dd>
<dd><b>Revised Release on the Pressure along the Melting and
Sublimation Curves of Ordinary Water Substance</b>. 2011 International Association for the Properties of Water and Steam
</dd>
</dl>
</html>"));
end IAPWS09;
