within MoMoLib.MoistAir.Utilities;
function f_pT "Enhancement factor as function of pressure and temperature"
  extends Modelica.Icons.Function;

  input Modelica.SIunits.AbsolutePressure p "Pressure";
  input Modelica.SIunits.Temperature T "Temperature";
  output Real f "Vapor-pressure enhancement factor";

protected
  function f_res
    extends MoMoLib.Common.Numerical.Interfaces.InputFunction;
    input Modelica.SIunits.AbsolutePressure p "Pressure";
    input Modelica.SIunits.Temperature T "Temperature";

  protected
    Real p_ws=0;
    Real kappa_T=0;
    Real v_ws=0;
    Real beta_H=0;
    Real psi_ws=0;
    Real baa=0;
    Real baw=0;
    Real bww=0;
    Real caaa=0;
    Real caaw=0;
    Real caww=0;
    Real cwww=0;
    final constant Real R_bar=8.314472;

  algorithm
  //p_ws is the saturation pressure of water
    p_ws := if (T >= 273.16) then Water.IAPWS95.Water95_Utilities.Basic.psat(T)
       else Water.IAPWS09.Ice09_Utilities.Basic.psub(T);

  //kappa_T is the isothermal compressibility
    if (p < p_ws) then
      kappa_T := 0;
    else
      kappa_T := if (T >= 273.16) then
        Modelica.Media.Water.IF97_Utilities.kappa_pT(p, T) else
        Water.IAPWS09.Ice09_Utilities.kappa_pT(p, T);
    end if;

  //v_ws is the molar volume of saturated water
    v_ws := if (T >= 273.16) then MoMoLib.MoistAir.Utilities.IF97_new.molarMass
      /Modelica.Media.Water.IF97_Utilities.rho_pT(p, T) else Water.IAPWS09.waterConstants.molarMass
      /Water.IAPWS09.Ice09_Utilities.rho_pT(p, T);

  //beta_H is Henry's law constant
    beta_H := MoMoLib.MoistAir.Utilities.beta_H(p, T);

  //calculating the virial coefficients baa, baw, bww, caaa, caaw, caww, cwww
    baa := MoMoLib.MoistAir.Utilities.VirialCoefficients.Baa_dT(0, T);
    baw := MoMoLib.MoistAir.Utilities.VirialCoefficients.Baw_dT(0, T);
    bww := MoMoLib.MoistAir.Utilities.VirialCoefficients.Bww_dT(0, T);
    caaa := MoMoLib.MoistAir.Utilities.VirialCoefficients.Caaa_dT(0, T);
    caaw := MoMoLib.MoistAir.Utilities.VirialCoefficients.Caaw_dT(0, T);
    caww := MoMoLib.MoistAir.Utilities.VirialCoefficients.Caww_dT(0, T);
    cwww := MoMoLib.MoistAir.Utilities.VirialCoefficients.Cwww_dT(0, T);

    y := ((1 + kappa_T*p_ws)*(p - p_ws) - kappa_T*(p^2 - p_ws^2)/2)/(R_bar*T)*
      v_ws + log(1 - beta_H*(1 - x*p_ws/p)*p) + (1 - x*p_ws/p)^2*p/(R_bar*T)*
      baa - 2*(1 - x*p_ws/p)^2*p/(R_bar*T)*baw - (p - p_ws - (1 - x*p_ws/p)^2*p)
      /(R_bar*T)*bww + (1 - x*p_ws/p)^3*p^2/(R_bar*T)^2*caaa + 3*(1 - x*p_ws/p)
      ^2*(1 - 2*(1 - x*p_ws/p))*p^2/(2*(R_bar*T)^2)*caaw - 3*(1 - x*p_ws/p)^2*x
      *p_ws/p*p^2/(R_bar*T)^2*caww - ((3 - 2*x*p_ws/p)*(x*p_ws/p)^2*p^2 - p_ws^
      2)/(2*(R_bar*T)^2)*cwww - (1 - x*p_ws/p)^2*(-2 + 3*x*p_ws/p)*x*p_ws/p*p^2
      /(R_bar*T)^2*baa*bww - 2*(1 - x*p_ws/p)^3*(-1 + 3*x*p_ws/p)*p^2/(R_bar*T)
      ^2*baa*baw + 6*(1 - x*p_ws/p)^2*(x*p_ws/p)^2*p^2/(R_bar*T)^2*bww*baw - 3*
      (1 - x*p_ws/p)^4*p^2/(2*(R_bar*T)^2)*baa^2 - 2*(1 - x*p_ws/p)^2*x*p_ws/p*
      (-2 + 3*x*p_ws/p)*p^2/(R_bar*T)^2*baw^2 - (p_ws^2 - (4 - 3*x*p_ws/p)*(x*
      p_ws/p)^3*p^2)/(2*(R_bar*T)^2)*bww^2 - log(x);
  end f_res;
  Real xmax;

algorithm
  if (T >= Water.IAPWS95.Water95_Utilities.Basic.Tsat(p)) then
    f := 1;
  else
    xmax := if (T < 273.16) then 120.0 else 8.0;
    f := MoMoLib.Common.Numerical.RootFinding.BrentMethod(
      function f_res(p=p, T=T),
      0.99999,
      xmax,
      1e-9);
  end if;
end f_pT;
