within MoMoLib;
package MoistAir "Air: Detailed moist air model (143.15 ... 2000 K)"


extends MoMoLib.Interfaces.PartialRealCondensingGases(
  mediumName="Moist air",
  substanceNames={"Water","Air"},
  final fixedX=false,
  final reducedX=true,
  final singleState=false,
  reference_X={0.01,0.99},
  fluidConstants={Utilities.Water.IAPWS95.waterConstants,MoMoLib.Air.dryairConstants},
  ThermoStates=Choices.IndependentVariables.pTX);

constant Integer Water=1
  "Index of water (in substanceNames, massFractions X, etc.)";

constant Integer Air=2
  "Index of air (in substanceNames, massFractions X, etc.)";

constant Real k_mair=steam.MM/dryair.MM "ratio of molar weights";

constant MoMoLib.Air.Air_Base.Constants dryair;
constant Utilities.Water.IAPWS95.Water95_Base.Constants steam;
constant SI.MolarMass[2] MMX={steam.MM,dryair.MM} "Molar masses of components";

import Modelica.Media.Interfaces;
import Modelica.Math;
import SI = Modelica.SIunits;
import Cv = Modelica.SIunits.Conversions;
import Modelica.Constants;
import Modelica.Media.IdealGases.Common.SingleGasNasa;


  redeclare record extends ThermodynamicState
  "ThermodynamicState record for moist air"
  end ThermodynamicState;


  redeclare replaceable model extends BaseProperties(
  T(stateSelect=if preferredMediumStates then StateSelect.prefer else
        StateSelect.default),
  p(stateSelect=if preferredMediumStates then StateSelect.prefer else
        StateSelect.default),
  Xi(stateSelect=if preferredMediumStates then StateSelect.prefer else
        StateSelect.default),
  redeclare final constant Boolean standardOrderComponents=true)
  "Moist air base properties record"

  MassFraction x_water "Mass of total water/mass of dry air";
  Real phi "Relative humidity";

protected
  MassFraction X_liquid "Mass fraction of liquid or solid water";
  MassFraction X_steam "Mass fraction of steam water";
  MassFraction X_air "Mass fraction of air";
  MassFraction X_sat
    "Steam water mass fraction of saturation boundary in kg_water/kg_moistair";
  MassFraction x_sat
    "Steam water mass content of saturation boundary in kg_water/kg_dryair";
  AbsolutePressure p_steam_sat "Partial saturation pressure of steam";
  equation
  assert(T >= 143.15 and T <= 2000,
    "Temperature T is not in the allowed range 143.15 K <= (T =" + String(T) +
    " K) <= 2000 K required from medium model \"" + mediumName + "\".");

  MM = 1/(Xi[Water]/MMX[Water] + (1.0 - Xi[Water])/MMX[Air]);

  p_steam_sat = Utilities.pds_pT(p, T);
  X_sat = k_mair/(p/p_steam_sat - 1 + k_mair);
  X_liquid = Xi[Water] - X_sat;
  X_steam = Xi[Water] - X_liquid;
  X_air = 1 - Xi[Water];

  h = specificEnthalpy_pTX(
      p,
      T,
      Xi);
  R = dryair.R*(X_air/(1 - X_liquid)) + steam.R*X_steam/(1 - X_liquid);
  u = Utilities.u_pTX(
      p,
      T,
      Xi);
  d = Utilities.rho_pTX(
      p,
      T,
      Xi);
  state.p = p;
  state.T = T;
  state.X = X;

    // these x are per unit mass of DRY air!
  x_sat = k_mair*p_steam_sat/max(100*Constants.eps, p - p_steam_sat);
  x_water = Xi[Water]/max(X_air, 100*Constants.eps);
  phi = Utilities.phi_pTX(
      p,
      T,
      Xi);

  end BaseProperties;


  redeclare function extends setState_pTX
  "Return thermodynamic state as function of pressure p, temperature T and composition X"
  algorithm
  state := if size(X, 1) == nX then ThermodynamicState(
      p=p,
      T=T,
      X=X) else ThermodynamicState(
      p=p,
      T=T,
      X=cat(
        1,
        X,
        {1 - sum(X)}));
  end setState_pTX;


  redeclare function extends setState_phX
  "Return thermodynamic state as function of pressure p, specific enthalpy h and composition X"
  algorithm
  state := if size(X, 1) == nX then ThermodynamicState(
      p=p,
      T=Utilities.Inverses.T_phX(
        p,
        h,
        X),
      X=X) else ThermodynamicState(
      p=p,
      T=Utilities.Inverses.T_phX(
        p,
        h,
        X),
      X=cat(
        1,
        X,
        {1 - sum(X)}));
  end setState_phX;


  redeclare function extends setState_psX
  "Return thermodynamic state as function of pressure p, specific enthalpy h and composition X"
  algorithm
  state := if size(X, 1) == nX then ThermodynamicState(
      p=p,
      T=Utilities.Inverses.T_psX(
        p,
        s,
        X),
      X=X) else ThermodynamicState(
      p=p,
      T=Utilities.Inverses.T_psX(
        p,
        s,
        X),
      X=cat(
        1,
        X,
        {1 - sum(X)}));
  end setState_psX;


  redeclare function extends setState_dTX
  "Return thermodynamic state as function of density d, temperature T and composition X"
  algorithm
  state := if size(X, 1) == nX then ThermodynamicState(
      p=Utilities.Inverses.p_dTX(
        d,
        T,
        X),
      T=T,
      X=X) else ThermodynamicState(
      p=Utilities.Inverses.p_dTX(
        d,
        T,
        X),
      T=T,
      X=cat(
        1,
        X,
        {1 - sum(X)}));
  end setState_dTX;


  redeclare function extends setSmoothState
  "Return thermodynamic state so that it smoothly approximates: if x > 0 then state_a else state_b"
  algorithm
  state := ThermodynamicState(
      p=Modelica.Media.Common.smoothStep(
        x,
        state_a.p,
        state_b.p,
        x_small),
      T=Modelica.Media.Common.smoothStep(
        x,
        state_a.T,
        state_b.T,
        x_small),
      X=Modelica.Media.Common.smoothStep(
        x,
        state_a.X,
        state_b.X,
        x_small));
  end setSmoothState;

function Xsaturation
  "Return absolute humitity per unit mass of moist air at saturation as a function of the thermodynamic state record"
  input ThermodynamicState state "Thermodynamic state record";
  output MassFraction X_sat "Steam mass fraction of sat. boundary";
protected
  MassFraction[:] X;
algorithm
  X := massFractionSaturation(state);
  X_sat := X[1];
end Xsaturation;

function xsaturation
  "Return absolute humitity per unit mass of dry air at saturation as a function of the thermodynamic state record"
  input ThermodynamicState state "Thermodynamic state record";
  output MassFraction x_sat "Absolute humidity per unit mass of dry air";
algorithm
  x_sat := Utilities.xws_pT(state.p, state.T);
  assert(x_sat > -1,
    "Calculation of absolute humidity is meaningless\nfor input pressure p = "
     + String(state.p) + " Pa and temperature T = " + String(state.T) + " K.");
end xsaturation;

  redeclare function extends massFraction_pTphi
  "Return mass fractions as a function of pressure, temperature and relative humidity"
protected
  Real pds;

  algorithm
  assert(phi < 1.0 and phi > 0, "Illegal input phi = " + String(phi) +
    ". Relative humidity is only defined in the range\n 0 <= phi <= 1.0.");
  pds := Utilities.pds_pT(p, T);
  assert(pds > -1,
    "Calculation of mass fraction of steam is meaningless\nfor input pressure p = "
     + String(p) + " Pa and temperature T = " + String(T) + " K.");
  X := {phi*k_mair/(p/pds - phi),1 - phi*k_mair/(p/pds - phi)};
  end massFraction_pTphi;


  redeclare function extends massFractionSaturation
  "Return saturation mass fractions"
protected
  AbsolutePressure pds;
  algorithm
  pds := Utilities.pds_pT(state.p, state.T);
  Xsat := {k_mair/(state.p/pds - 1 + k_mair),(state.p/pds - 1)/(state.p/pds - 1
     + k_mair)};
  assert(Xsat[1] > -1,
    "Calculation of saturation mass fraction is meaningless\nfor input pressure p = "
     + String(state.p) + " Pa and temperature T = " + String(state.T) + " K.");
  end massFractionSaturation;


  redeclare function extends relativeHumidity "Return relativ humidity"
  algorithm
  phi := Utilities.phi_pTX(
      state.p,
      state.T,
      state.X);
  assert(phi > -1,
    "Calculation of relative humidity is meaningless\nfor input pressure p = "
     + String(state.p) + " Pa and temperature T = " + String(state.T) + " K.");
  end relativeHumidity;


  redeclare function extends gasConstant
  "Return ideal gas constant as a function from thermodynamic state, only valid for phi<1"

  algorithm
  R := dryair.R*(1 - state.X[Water]) + steam.R*state.X[Water];
  end gasConstant;


  redeclare function extends saturationPressure
  "Return saturation pressure of condensing fluid"
  algorithm
  psat := Utilities.pds_pT(state.p, state.T);
  assert(psat > -1,
    "Calculation of saturation pressure is meaningless\nfor input temperature T = "
     + String(state.T) + " K.");
  end saturationPressure;


  redeclare function extends saturationTemperature
  "Return saturation temperature of condensing fluid"
protected
    function Tsat_res
    extends MoMoLib.Common.Numerical.Interfaces.InputFunction;
    input SI.Pressure p "Pressure";

    algorithm
    y := MoMoLib.MoistAir.Utilities.pds_pT(p=p, T=x) - p;
    end Tsat_res;

  algorithm
  Tsat := MoMoLib.Common.Numerical.RootFinding.BrentMethod(
      function Tsat_res(state.p),
      50,
      673.15,
      1e-9);
  end saturationTemperature;


  redeclare function extends enthalpyOfVaporization
  "Return enthalpy of vaporization of water"
protected
  AbsolutePressure p_liq;
  algorithm
  p_liq := saturationPressureLiquid(state);
  r0 := Modelica.Media.Water.IF97_Utilities.hv_p(p_liq) -
    Modelica.Media.Water.IF97_Utilities.hl_p(p_liq);
  end enthalpyOfVaporization;


  redeclare function extends enthalpyOfLiquid "Return enthalpy of liquid water"
protected
  Real xw;
  Real xws;
  algorithm
  xw := state.X[1]/(1 - state.X[1]);
  xws := Utilities.xws_pT(state.p, state.T);
  if ((xws > xw) and (state.T > 273.15)) then
    h := Modelica.Media.Water.IF97_Utilities.h_pT(
        state.p,
        state.T,
        region=1);
  else
    h := 0;
  end if;
  end enthalpyOfLiquid;


  redeclare function extends enthalpyOfGas
  "Return specific enthalpy of gas (air and steam)"
protected
  Real xw;
  Real xws;
  Real pd;
  Real pl;

  algorithm
  pd := Utilities.pd_pTX(
      state.p,
      state.T,
      state.X);
  pl := state.p - pd;
  xw := state.X[1]/(1 - state.X[1]);
  xws := Utilities.xws_pT(state.p, state.T);
  if ((xw <= xws) or (xws == -1)) then
    h := MoMoLib.Air.Air_Utilities.h_pT(pl, state.T) + xw*
      Utilities.IF97_new.h_pT(pd, state.T);
  else
    if (state.T < 273.16) then
      h := MoMoLib.Air.Air_Utilities.h_pT(pl, state.T) + xws*
        Utilities.IF97_new.h_pT(pd, state.T);
    else
      h := MoMoLib.Air.Air_Utilities.h_pT(pl, state.T) + xws*
        Utilities.IF97_new.h_pT(pd, state.T);
    end if;
  end if;
  annotation (Inline=false, LateInline=true);
  end enthalpyOfGas;


  redeclare function extends enthalpyOfCondensingGas
  "Return specific enthalpy of steam"
protected
  Real xw;
  Real pd;

  algorithm
  pd := Utilities.pd_pTX(
      state.p,
      state.T,
      state.X);
  xw := state.X[1]/(1 - state.X[1]);
  h := xw*Utilities.IF97_new.h_pT(pd, state.T);
  annotation (Inline=false, LateInline=true);
  end enthalpyOfCondensingGas;


  redeclare function extends enthalpyOfNonCondensingGas
  "Return specific enthalpy of dry air"

  algorithm
  h := MoMoLib.Air.Air_Utilities.h_pT(state.p, state.T);
  end enthalpyOfNonCondensingGas;


  redeclare function extends pressure
  "Returns pressure of ideal gas as a function of the thermodynamic state record"

  algorithm
  p := state.p;

  end pressure;


  redeclare function extends temperature
  "Return temperature of ideal gas as a function of the thermodynamic state record"

  algorithm
  T := state.T;

  end temperature;


  redeclare function extends density
  "Returns density as a function of the thermodynamic state record"

  algorithm
  d := Utilities.rho_pTX(
      state.p,
      state.T,
      state.X);

  end density;


  redeclare function extends specificEnthalpy
  "Return specific enthalpy of moist air as a function of the thermodynamic state record"

  algorithm
  h := Utilities.h_pTX(
      state.p,
      state.T,
      state.X);

  end specificEnthalpy;


  redeclare function extends specificInternalEnergy
  "Return specific internal energy of moist air as a function of the thermodynamic state record"

  algorithm
  u := Utilities.u_pTX(
      state.p,
      state.T,
      state.X);

  end specificInternalEnergy;


  redeclare function extends specificEntropy
  "Return specific entropy from thermodynamic state record, only valid for phi<1"

  algorithm
  s := Utilities.s_pTX(
      state.p,
      state.T,
      state.X);

  end specificEntropy;


  redeclare function extends specificGibbsEnergy
  "Return specific Gibbs energy as a function of the thermodynamic state record, only valid for phi<1"
  extends Modelica.Icons.Function;
  algorithm
  g := Utilities.h_pTX(
      state.p,
      state.T,
      state.X) - state.T*Utilities.s_pTX(
      state.p,
      state.T,
      state.X);

  end specificGibbsEnergy;


  redeclare function extends specificHelmholtzEnergy
  "Return specific Helmholtz energy as a function of the thermodynamic state record, only valid for phi<1"

  algorithm
  f := Utilities.u_pTX(
      state.p,
      state.T,
      state.X) - state.T*Utilities.s_pTX(
      state.p,
      state.T,
      state.X);

  end specificHelmholtzEnergy;


  redeclare function extends specificHeatCapacityCp
  "Return specific heat capacity at constant pressure as a function of the thermodynamic state record"

  algorithm
  cp := Utilities.cp_pTX(
      state.p,
      state.T,
      state.X);

  end specificHeatCapacityCp;


  redeclare function extends specificHeatCapacityCv
  "Return specific heat capacity at constant volume as a function of the thermodynamic state record"

  algorithm
  cv := Utilities.cv_pTX(
      state.p,
      state.T,
      state.X);

  end specificHeatCapacityCv;


  redeclare function extends isentropicExponent "Return isentropic exponent"
  algorithm
  gamma := specificHeatCapacityCp(state)/specificHeatCapacityCv(state);
  end isentropicExponent;


  redeclare function extends isentropicEnthalpy "Return isentropic enthalpy"
protected
  MassFraction[nX] X "complete X-vector";
  algorithm
  X := if reducedX then cat(
      1,
      refState.X,
      {1 - sum(refState.X)}) else refState.X;
  h_is := specificEnthalpy(setState_psX(
      p_downstream,
      specificEntropy(refState),
      X));
  end isentropicEnthalpy;


  redeclare function extends velocityOfSound "Return velocity of sound"
  algorithm
  a := sqrt(max(0, gasConstant(state)*state.T*specificHeatCapacityCp(state)/
    specificHeatCapacityCv(state)));
  end velocityOfSound;


  redeclare function extends molarMass "Return the molar mass of the medium"
  algorithm
  MM := 1/(state.X[1]*steam.MM + state.X[2]*dryair.MM);
  end molarMass;


  redeclare function extends dynamicViscosity
  "Return dynamic viscosity as a function of the thermodynamic state record, valid from 73.15 K to 373.15 K"
  algorithm
  eta := Utilities.Transport.eta_pTX(
      state.p,
      state.T,
      state.X);

  end dynamicViscosity;


  redeclare function extends thermalConductivity
  "Return thermal conductivity as a function of the thermodynamic state record, valid from 73.15 K to 373.15 K"
  algorithm
  lambda := Utilities.Transport.lambda_pTX(
      state.p,
      state.T,
      state.X);

  end thermalConductivity;


annotation (Documentation(info="<html>
<p>
Calculation of fluid properties for moist air in the region from 143.15 Kelvin to 2000 Kelvin at pressures up to 10 MPa. This model of moist air is based on
the diploma thesis of Hellriegel [10] with small modifications. Moist air is treated as an ideal mixture of the real fluids air and water.
</p>

<h4>Restriction</h4>
<p>
The functions provided by this package shall be used inside of the restricted limits according to the referenced literature.
</p>

<ul>
<li>
<b>611.2 Pa &le; p &le; 10 MPa</b>
</li>
<li>
<b>143.15 K &le; T &le; 2000 K</i>
</li>
</ul>


<h4>Calculation algorithms</h4>
<h5>Nomenclature</h5>
<table width=\"25%\">
<colgroup>
<col width=\"1*\">
<col width=\"2*\">
</colgroup>
<tr>
<td>p</td>
<td>Mixture pressure in Pa</td>
</tr>
<tr>
<td>T</td>
<td>Temperature in K</td>
</tr>
<tr>
<td>x<sub>w</sub></td>
<td>Absolutue humidity in kg(water)/kg(dry air)</td>
</tr>
<tr>
<td>x<sub>ws</sub></td>
<td>Absolutue humidity on saturation boundary in kg(water)/kg(dry air)</td>
</tr>
<tr>
<td>&phi;</td>
<td>Relative humidity (only defined for unsaturated humid air)</td>
</tr>
</table>

<h5>Unsaturated and saturated humid air (0 &le; x<sub>w</sub> &le; x<sub>ws</sub>)</h5>
<p>
Ideal mixture of dry air and steam
<ul>
<li>Dry air:</li>
<ul>
<li>d,h,u,s,c<sub>p</sub> from [1]</li>
<li>&lambda;, &eta; from [2]</li>
</ul>
<li>Steam:</li>
<ul>
<li>d,h,u,s,c<sub>p</sub> from [4]</li>
<li>&lambda;, &eta; for 273.15 K &le; T &le; 1073.15 K from [5] and [6]</li>
<li>&lambda;, &eta; for T &lt; 273.15 K or T &gt; 1073.15 K from [12]</li>
</ul>
</ul>
</p>

<h5>Supersaturated humid air (liquid fog and ice fog)</h5>
<p>
<b>Liquid fog (x<sub>w</sub> &gt; x<sub>wsw</sub>) and T &ge; 273.16 K</b><br />
Ideal mixture of saturated humid air and water
<ul>
<li>Saturated humid air (see above)</li>
<li>d,h,u,s of liquid droplets from [4]</li>
<li>c<sub>p</sub> is not defined</li>
<li>&lambda;, &eta; of liquid droplets from [5] and [6]</li>
</ul>
</p>
<p>
<b>Ice fog (x<sub>w</sub> &gt; x<sub>wsw</sub>) and T &lt; 273.16 K</b><br />
Ideal mixture of saturated humid air and ice
<ul>
<li>Saturated humid air (see above)</li>
<li>d,h,u,s of ice crystals from [7]</li>
<li>c<sub>p</sub> is not defined</li>
<li>&lambda; of ice as constant value</li>
<li>&eta; of ice is neglected</li>
</ul>
</p>

<h5>Saturation pressure of water in moist air</h5>
<p>
The saturation pressure p<sub>ds</sub> of water in moist air is calculated by p<sub>ds</sub> = f*p<sub>sat</sub>, where
<ul>
<li>f is the enhancement factor from [9] and [3]</li>
<li>p<sub>sat</sub> for T &ge; 273.16 K is the saturation pressure from [4]</li>
<li>p<sub>sat</sub> for T &lt; 273.16 K is the saturation pressure from [8]</li>
</ul>
</p>

<h5>Dissociation</h5>
<p>
For temperatures above 773.15 K effects of dissociation are taken into account. Dissociation is modeled according to [11].
For high temperatures the calculated values for moist air with 0 kg(water)/kg(dry air) (i.e. dry air) may differ from those
calculated by the package MoMoLib.Air, because there no dissociation is considered.
</p>

<h4>References</h4>
<dl>
<dd>[1] <b>Thermodynamic Properties of Air and Mixtures of Nitrogen, Argon,
and Oxygen From 60 to 2000 K at Pressures to 2000 MPa</b>. J. Phys. Chem. Ref. Data, Vol. 29, No. 3, 2000.
</dd>
<dd>[2] <b>Viscosity and Thermal Conductivity Equations for
Nitrogen, Oxygen, Argon, and Air</b>. International Journal of Thermophysics, Vol. 25, No. 1, January 2004
</dd>
<dd>[3] <b>Revised Release on the IAPWS Formulation 1995 for the Thermodynamic
Properties of Ordinary Water Substance for General and Scientific Use</b>. 2009 International Association for the Properties of Water and Steam.
</dd>
<dd>[4] <b>Revised Release on the IAPWS Industrial Formulation 1997
for the Thermodynamic Properties of Water and Steam</b>. 2007 International Association for the Properties of Water and Steam.
</dd>
<dd>[5] <b>Release on the IAPWS Formulation 2008 for the Viscosity of Ordinary Water Substance</b>. 2008 International Association for the Properties of Water and Steam
</dd>
<dd>[6] <b>Release on the IAPWS Formulation 2011 for the Thermal Conductivity of
Ordinary Water Substance</b>. 2011 International Association for the Properties of Water and Steam.
</dd>
<dd>[7] <b>Revised Release on the Equation of State 2006 for H2O Ice Ih</b>. 2009 International Association for the Properties of Water and Steam.
</dd>
<dd>[8] <b>Revised Release on the Pressure along the Melting and
Sublimation Curves of Ordinary Water Substance</b>. 2011 International Association for the Properties of Water and Steam.
</dd>
<dd>[9] <b>Determination of Thermodynamic and Transport Properties
of Humid Air for Power-Cycle Calculations</b>. 2009 PTB, Braunschweig, Germany.
</dd>
<dd>[10] <b>Berechnung der thermodynamischen Zustandsfunktionen von
feuchter Luft in energietechnischen Prozessmodellierungen</b>. 2001 Diplomarbeit, Zittau.
</dd>
<dd>[11] <b>Thermodynamische Stoffwerte von feuchter Luft und Verbrennungsgasen</b>. 2003 VDI-Richtlinie 4670.
</dd>
<dd>[12] <b>W&auml;rme&uuml;bertragung in Dampferzeugern und W&auml;rmetauschern</b>. 1985 FDBR-Fachbuchreihe, Bd. 2, Vulkan Verlag Essen.
</dd>
</dl>
</html>"));
end MoistAir;
