within MoMoLib.R134a.R134a_ph;
function dt_ph "density and temperature w.r.t. pressure and specific enthalpy"

  extends Modelica.Icons.Function;

  input AbsolutePressure p "pressure";
  input SpecificEnthalpy h "specific enthalpy";
  output Density d "density";
  output Temperature T "temperature";
protected
  SaturationProperties sat(psat=p, Tsat=0)
    "saturation temperature and pressure";
  Modelica.SIunits.Pressure delp=1.0e-2 "relative error in p in iteration";
  Modelica.SIunits.SpecificEnthalpy delh=1.0e-2
    "relative error in h in iteration";
  Modelica.SIunits.SpecificEnthalpy hvapor=
      dewEnthalpy(sat=sat) "vapor enthalpy";
  Modelica.SIunits.SpecificEnthalpy hliquid=
      bubbleEnthalpy(sat=sat) "liquid enthalpy";
  Integer error "iteration error";
  Real x "vapor quality";
  Real vvap "specific volume vapor";
  Real vliq "specific volume liquid";
algorithm
  if ((h < hliquid) or (h > hvapor) or (p > R134aData.data.FPCRIT)) then
    (d,T,error) := MoMoLib.R134a.R134a_ph.dtofphOnePhase(
                          p, h, delp, delh);
  else
    vvap := 1/dewDensity(sat=sat);
    vliq := 1/bubbleDensity(sat=sat);
    x := (h - hliquid)/(hvapor - hliquid);
    error := 0;
    T := saturationTemperature(p);
    d := 1/(vliq + x*(vvap - vliq));
  end if;

  annotation ( Documentation(info="<html>
<p> This function calculates the density and temperature of R134a from absolute pressure and specific enthalpy. In one-phase region the function calls the fundamental helmholtz equation of Tillner-Roth (1994). In two-phase the density and temperature is computed from cubic splines for saturated pressure, liquid and vapor density.
</p>
<h4>Restrictions</h4>
The function cannot be inverted in a numerical way. Please use functions <a href=\"modelica://MoMoLib.R134a.R134a_ph.rho_props_ph\">rho_props_ph</a> and <a href=\"modelica://MoMoLib.R134a.R134a_ph.T_props_ph\">T_props_ph</a> for this purpose.
</html>"));
end dt_ph;
