within MoMoLib.R134a.R134a_ph;
function dtofphOnePhase
  "density and temperature w.r.t. pressure and specific enthalpy in one-phase region"

  extends Modelica.Icons.Function;

  input AbsolutePressure p "pressure";
  input SpecificEnthalpy h "enthalpy";
  input AbsolutePressure delp "absolute error in p in iteration";
  input SpecificEnthalpy delh "absolute error in h in iteration";

  output Density d "density";
  output Temperature T "temperature";
  output Integer error "1 if did not converged";

protected
  constant Real p_breaks[:]=MoMoLib.R134a.R134aData.pbreaks
    "grid points of reduced pressure";
  constant Real hl_coef[:, 4]=MoMoLib.R134a.R134aData.hlcoef
    "coefficients of cubic spline for h_liq(p)";
  constant Real dl_coef[:, 4]=MoMoLib.R134a.R134aData.dlcoef
    "coefficients of cubic spline for rho_liq(p)";
  constant Real T_coef[:, 4]=MoMoLib.R134a.R134aData.Tcoef
    "coefficients of cubic spline for Tsat(p)";
  constant Real dv_coef[:, 4]=MoMoLib.R134a.R134aData.dvcoef
    "coefficients of cubic spline for rho_vap(p)";

  Modelica.SIunits.SpecificEnthalpy hl "liquid enthalpy";
  Boolean liquid "is liquid";
  Boolean supercritical "is supercritcal";
  Integer int "interval number";
  Real pred "reduced pressure";
  Real localx "oordinate of local spline";
  Integer i "newton iteration number";
  Real dh "difference in h";
  Real dp "difference in p";
  Real damping "damping constant";
  Real det "determinante";
  Real deld "density change";
  Real delt "temperature change";
  Common.HelmholtzDerivs f "helmholtz derivatives";
  Common.NewtonDerivatives_ph nDerivs "newton derivatives";
  Boolean found "iteration converged";
algorithm
  i := 0;
  error := 0;
  found := false;
  damping := 1.0;
                  //by default no damping
  pred := p/R134aData.data.FPCRIT;
  (int,error) := Common.FindInterval(pred, p_breaks);
  localx := pred - p_breaks[int];
  // set decent initial guesses for d and T
  supercritical := p > R134aData.data.FPCRIT;
  if supercritical then
    // iteration seems to work better if coming from high densities
    d := R134aData.data.FDCRIT*3.0;
    //      dTR.T := data.TCRIT + 10.0;
    // linear approximation that can be made generic
    T := 270.0 + 120.0*(h/R134aData.data.HCRIT);
  else
    liquid := h < 1.01*R134aData.data.HCRIT*
      Common.CubicSplineEval(localx, hl_coef[int, 1:4]);
    if liquid then
      damping := 0.3;
              //damping on the liquid side
      d := R134aData.data.FDCRIT*
        Common.CubicSplineEval(localx, dl_coef[int, 1:4])*1.02;
      T := Common.CubicSplineEval(localx, T_coef[int, 1:4]);
    else
      d := R134aData.data.FDCRIT*
        Common.CubicSplineEval(localx, dv_coef[int, 1:4])*0.95;
      T := Common.CubicSplineEval(localx, T_coef[int, 1:4]);
    end if;
  end if;

  // the Newton iteration
  while ((i < 50) and not found) loop
    f := f_R134a(d, T);
    // nDerivs are the symbolic derivatives needed in the iteration
    // for given d and T
    nDerivs := MoMoLib.R134a.Common.Helmholtz_ph(
                                    f);
    dh := nDerivs.h - h;
    dp := nDerivs.p - p;
    if ((abs(dh) <= delh) and (abs(dp) <= delp)) then
      found := true;
      error := if not found then 1 else 0;
    end if;
    det := nDerivs.ht*nDerivs.pd - nDerivs.pt*nDerivs.hd;
    if abs(det) < 1e-10 then
      det := sign(det)*1e-10;
    end if;
    delt := (nDerivs.pd*dh - nDerivs.hd*dp)/det;
    deld := (nDerivs.ht*dp - nDerivs.pt*dh)/det;
    T := T - damping*delt;
    d := d - damping*deld;
    // Check for limit of state variables
    if T < R134aData.triple.TTRIPLE then
      T := R134aData.triple.TTRIPLE + 0.1;
    end if;
    if d <= 0.0 then
      d := R134aData.triple.DVTRIPLE;
    elseif d > 1.5*R134aData.triple.DLTRIPLE then
      d := R134aData.triple.DLTRIPLE;
    end if;
    i := i + 1;
  end while;

  annotation ( Documentation(info="<html>
<p> This function calculates the density and temperature of R134a from absolute pressure and specific enthalpy in one-phase region. The function calls the fundamental helmholtz equation of Tillner-Roth (1994) which is requiring density and temperature for input. Thus, a newton iteration is performed to determine density and temperature. The newton iteration stops if the inputs for pressure difference delp and specific enthalpy difference delh are larger than the actual differences derived from the newton iteration.
</p>
<h4>Restrictions</h4>
The function shall only be used for one-phase inputs since the fundamental equation is not valid for two-phase states.
</html>"));
end dtofphOnePhase;
