within MoMoLib.R134a.R134a_ph;
function dtofpsOnePhase "inverse iteration in one phase region (d,T) = f(p,s)"
  input AbsolutePressure p "pressure";
  input SpecificEntropy s "specific entropy";
  input AbsolutePressure delp "absolute iteration accuracy";
  input SpecificEntropy dels "absolute iteration accuracy";
  output Density d "density";
  output Temperature T "temperature";
  output Integer error "error flag: trouble if different from 0";

protected
  constant Real p_breaks[:] = MoMoLib.R134a.R134aData.pbreaks
    "grid points of reduced pressure";
  constant Real dl_coef[:, 4]=MoMoLib.R134a.R134aData.dlcoef
    "coefficients of cubic spline for rho_liq(p)";
  constant Real dv_coef[:, 4]=MoMoLib.R134a.R134aData.dvcoef
    "coefficients of cubic spline for rho_vap(p)";
  constant Real sl_coef[:, 4] = MoMoLib.R134a.R134aData.slcoef
    "coefficients of cubic spline for s_liq(p)";
  constant Real sv_coef[:, 4] = MoMoLib.R134a.R134aData.svcoef
    "coefficients of cubic spline for s_vap(p)";
  constant Real T_coef[:, 4]=MoMoLib.R134a.R134aData.Tcoef
    "coefficients of cubic spline for Tsat(p)";
  Integer i "newton iteration number";
  Real ds "difference in s";
  Real dp "difference in p";
  Real det "determinante";
  Real deld "density change";
  Real delt "temperature change";
  Common.HelmholtzDerivs f
    "dimensionless Helmholtz function and dervatives w.r.t. dimensionless d and T";
  Common.NewtonDerivatives_ps nDerivs "newton derivatives";
  Boolean liquid "is liquid";
  Boolean supercritical "is supercritcal";
  Boolean found "iteration converged";
  Integer int "interval number";
  Real pred "reduced pressure";
  Real localx "oordinate of local spline";
  Real sl "liquid entropy";
  Real sv "vapor entropy";
algorithm
  i := 0;
  error := 0;
  found := false;
  pred := p/R134aData.data.FPCRIT;
  (int,error) := Common.FindInterval(pred,
    p_breaks);
  localx := pred - p_breaks[int];
  // set decent initial guesses for d and T
  supercritical := p > R134aData.data.FPCRIT;
  if supercritical then
    d := R134aData.data.FDCRIT + 500.0;
    T := R134aData.data.FTCRIT + 10.0;
  else
    sl := R134aData.data.SCRIT*
      Common.CubicSplineEval(localx,
      sl_coef[int, 1:4]);
    sv := R134aData.data.SCRIT*
      Common.CubicSplineEval(localx,
      sv_coef[int, 1:4]);
    liquid := s < sl;
    if liquid then
      d := R134aData.data.FDCRIT*
        Common.CubicSplineEval(localx,
        dl_coef[int, 1:4])*1.1;//1.05
      T := Common.CubicSplineEval(localx,
         T_coef[int, 1:4])*0.9;//0.95
    else
      d := R134aData.data.FDCRIT*
        Common.CubicSplineEval(localx,
        dv_coef[int, 1:4])*0.8; //0.9
      T := Common.CubicSplineEval(localx,
         T_coef[int, 1:4])*1.1 + 0.5*(s - sv);
    end if;
  end if;

  while ((i < 100) and not found) loop
    f := f_R134a(d=d, T=T);
    nDerivs := MoMoLib.R134a.Common.Helmholtz_ps(
                                    f);
    ds := nDerivs.s - s;
    dp := nDerivs.p - p;
    if ((abs(ds/s) <= dels) and (abs(dp/p) <= delp)) then
      found := true;
    end if;
    det := nDerivs.st*nDerivs.pd - nDerivs.pt*nDerivs.sd;
    delt := (nDerivs.pd*ds - nDerivs.sd*dp)/det;
    deld := (nDerivs.st*dp - nDerivs.pt*ds)/det;
    T := T - delt;
    d := d - deld;
    i := i + 1;
  end while;
  if not found then
    error := 1;
  end if;

  annotation (Documentation(info="<html>
<p> This function calculates the density and temperature of R134a from absolute pressure and specific entropy in one-phase region. The function calls the fundamental helmholtz equation of Tillner-Roth (1994) which is requiring density and temperature for input. Thus, a newton iteration is performed to determine density and temperature. The newton iteration stops if the inputs for pressure difference delp and specific entropy difference dels are larger than the actual differences derived from the newton iteration.
</p>
<h4>Restrictions</h4>
The function shall only be used for one-phase inputs since the fundamental equation is not valid for two-phase states. The iteration could fail for liquid states with high pressures.
</html>"));
end dtofpsOnePhase;
