within MoMoLib.R134a;
package R134a_ph "medium model for R134a and p,h as states"


  extends Modelica.Media.Interfaces.PartialTwoPhaseMedium(
     ThermoStates = Choices.IndependentVariables.ph,
     mediumName="R134a_ph",
     substanceNames={"tetrafluoroethan"},
     singleState=false,
     SpecificEnthalpy(start=h_default, nominal=5.0e5),
     Density(start=4, nominal=500),
     AbsolutePressure(start=p_default, nominal=10e5),
     Temperature(start=T_default, nominal=350),
     smoothModel=false,
     onePhase=false,
     fluidConstants = r134aConstants,
     h_default = 420e3);
     //Exchange for MSL 3.2.1:ThermoStates = Modelica.Media.Interfaces.Choices.IndependentVariables.ph,
  constant Boolean ph_explicit=true;
  constant Boolean dT_explicit=false;
  constant FluidLimits[1] r134aLimits(
    each TMIN = 169.85,
    each TMAX = 455,
    each DMIN = 0,
    each DMAX = 1591.7,
    each PMIN = 389.563789,
    each PMAX = 7e7,
    each HMIN = 0,
    each HMAX = 0,
    each SMIN = 0,
    each SMAX = 0);
  constant FluidConstants[1] r134aConstants(
     each chemicalFormula = "CF3CH2F",
     each structureFormula="1,1,1,2-tetrafluoroethane",
     each iupacName="tetrafluoroethan",
     each casRegistryNumber="811-97-2",
     each molarMass=R134aData.data.MM,
     each criticalMolarVolume=R134aData.data.MM/R134aData.data.DCRIT,
     each criticalTemperature=R134aData.data.TCRIT,
     each criticalPressure=R134aData.data.PCRIT,
     each triplePointTemperature=R134aData.data.TTRIPLE,
     each triplePointPressure=R134aData.data.PTRIPLE,
     each meltingPoint=172.15,
     each normalBoilingPoint=247.076,
     each acentricFactor=0.32684,
     each dipoleMoment=1.99,
     each hasCriticalData=true);


  redeclare record extends SaturationProperties
  end SaturationProperties;


  redeclare record extends ThermodynamicState "thermodynamic state"
    SpecificEnthalpy h "specific enthalpy";
    Density d "density";
    Temperature T "temperature";
    AbsolutePressure p "pressure";

  end ThermodynamicState;


  redeclare replaceable model extends BaseProperties(
    h(stateSelect=StateSelect.prefer),
    d(stateSelect=StateSelect.default),
    T(stateSelect=StateSelect.default),
    p(stateSelect=StateSelect.prefer)) "Base properties of R134a"
    Integer phase(min=0, max=2, start=1,fixed=false)
    "2 for two-phase, 1 for one-phase, 0 if not known";
    SaturationProperties sat(Tsat(start=273.0), psat(start=3.0e5))
    "saturation temperature and pressure";
    MassFraction quality "quality of vapour";

  equation
    MM = R134aData.data.MM;
    phase = 0;

    T = temperature_ph(p, h);
    d = density_ph(p, h);
    sat.Tsat = saturationTemperature(p=p);
    sat.psat = p;
    quality = vapourQuality(state);

    u = h - p/d;
    R = R134aData.data.R;
    h = state.h;
    p = state.p;
    T = state.T;
    d = state.d;
    phase = state.phase;
  end BaseProperties;


  redeclare function extends setState_phX
  "set state for pressure and specific enthalpy (X not used since single substance)"

protected
   SaturationProperties sat(psat=p, Tsat=0)
    "saturation temperature and pressure";
   Modelica.SIunits.SpecificEnthalpy hl=bubbleEnthalpy(sat) "liquid enthalpy";
   Modelica.SIunits.SpecificEnthalpy hv=dewEnthalpy(sat) "vapor enthalpy";

  algorithm
  state.p :=p;
  state.h :=h;

  state.phase := if ((state.h < hl) or (state.h > hv) or (state.p > R134aData.data.FPCRIT)) then 1 else 2;

  state.d := density_ph(p, h);
  state.T := temperature_ph(p, h);
    annotation ( Documentation(info="<html>
<p>This function should be used by default in order to calculate the thermodynamic state record used as input by many functions.</p>
<p>
Example:
<pre>
     parameter Medium.AbsolutePressure p = 3e5;
     parameter Medium.SpecificEnthalpy h = 4.2e5;

     Medium.Density rho;

     <b>equation</b>

     rho = Medium.density(setState_phX(p, h, fill(0, Medium.nX)));
</pre>
</p>

</html>"));
  end setState_phX;


  redeclare function extends setState_dTX
  "set state for density and temperature (X not used since single substance)"
protected
    Common.HelmholtzDerivs f "helmholtz derivatives";
    Modelica.SIunits.SpecificHeatCapacity R "specific gas constant";
    SaturationProperties sat "saturation temperature and pressure";
    Modelica.SIunits.Density dl "liquid density";
    Modelica.SIunits.Density dv "vapor density";

  algorithm
    R := R134aData.data.R;
    f := f_R134a(d=d, T=T);
    state.p := d*R*T*f.delta*f.fdelta;
    state.h := R*T*(f.tau*f.ftau + f.delta*f.fdelta);
    sat.psat := state.p;
    state.T := T;
    state.d := d;
    sat.Tsat := state.T;
    dl := bubbleDensity(sat);
    dv := dewDensity(sat);
    state.phase := if ((state.d < dl) or (state.d > dv) or (state.p > R134aData.data.FPCRIT)) then 1 else 2;

     annotation ( Documentation(revisions="<html>
<p>2012-08-01  Stefan Wischhusen: Corrected passing-error of inputs.</p>
</html>",   info="<html>
<p>Although the medium package is explicit for pressure and specific enthalpy, this function may be used in order to calculate the thermodynamic state record used as input by many functions. It will calculate the missing states:</p>
<ul>
<li>pressure</li>
<li>specific enthalpy</li>
</ul>
<p>
Example:
<pre>
     parameter Medium.Density d = 4;
     parameter Medium.Temperature T = 298;

     Medium.SpecficEntropy s;

     <b>equation</b>

     s = Medium.specificEntropy(setState_dTX(d, T, fill(0, Medium.nX)));
</pre>
</p>
</html>"));
  end setState_dTX;


  redeclare function extends setState_psX
  "set state for pressure and specific entropy (X not used since single substance)"

protected
    Modelica.SIunits.Pressure delp=1e-2 "iteration accuracy for pressure";
    Modelica.SIunits.SpecificEntropy dels=1e-1 "iteration accuracy for entropy";
    Integer error "if newton iteration fails (too many calls)";
    Common.HelmholtzDerivs f "helmholtz derivatives";
    SaturationProperties sat "saturation temperature and pressure";
  algorithm
    state.p := p;

    if (getPhase_ps(p, s) == 1) then
      (state.d,state.T,error) := dtofpsOnePhase(p=p, s=s, delp=delp, dels=dels);
      f := f_R134a(d=state.d, T=state.T);
      state.h := R134aData.R*state.T*(f.tau*f.ftau + f.delta*f.fdelta);
    else
      state.h := hofpsTwoPhase(p, s);
    end if;

    annotation ( Documentation(info="<html>
<p>This function may be used in order to calculate the thermodynamic state record used as input by many functions. It will calculate the missing states:</p>
<ul>
<li>density</li>
<li>pressure</li>
<li>specific enthalpy</li>
</ul>
<p>
Example:
<pre>
     parameter Medium.AbsolutePressure p = 3e5;
     parameter Medium.SpecficEntropy s = 1.7e3;

     Medium.SpecficEnthalpy h;

     <b>equation</b>

     h = Medium.specificEnthalpy(setState_psX(p, s, fill(0, Medium.nX)));
</pre>
</p>
</html>"));
  end setState_psX;


  redeclare function extends setState_pTX "dummy function"
  input FixedPhase phase=0 "number of phases";
  algorithm
   // Will probably result in an error so that the user has a chance to fix the model
   // With pressure and temperature a calculation of two-phase properties is not possible
   // Please use density and temperature or better pressure and specific enthalpy as states
   state.d := 0;
   state.h := 0;
     annotation ( Documentation(info="<html>
<p>This function is intentionally provided although it just calculates dummy values. Please note, that pressure and temperature cannot be used to determine states in two-phase region. Therefore, in order to avoid wrong usage, the function will set the remaining states to zero.</p>
</html>"));
  end setState_pTX;


  redeclare function extends setBubbleState
  "Return the thermodynamic state on the bubble line"
  algorithm
    if sat.psat<MoMoLib.R134a.R134aData.data.FPCRIT then
    state.p :=sat.psat;
    state.T :=saturationTemperature(sat.psat);
    state.d :=bubbleDensity(sat);
    state.h :=bubbleEnthalpy(sat);
    else
      assert(sat.psat<MoMoLib.R134a.R134aData.data.FPCRIT, "function setBubbleState is only valid in two-phase regime");
    end if;
    annotation ( Documentation(info="<html>
<p>This function shall be used in order to calculate the thermodynamic state record for the liquid phase boundary. It requires the saturation record as input which can be determined by both functions setSat_p and setSat_T:
</p>
<p>
Example:
<pre>
    Medium.AbsolutePressure p=3e5;
    // Viscosity on the liquid phase boundary
    Modelica.SIunits.DynamicViscosity eta_liq;

    equation

    eta_liq = Medium.DynamicViscosity(Medium.setBubbleState(Medium.setSat_p(p)));
</pre>

<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end setBubbleState;


  redeclare function extends setDewState
  "Return the thermodynamic state on the dew line"
  algorithm
    if sat.psat<MoMoLib.R134a.R134aData.data.FPCRIT then
    state.p :=sat.psat;
    state.T :=saturationTemperature(sat.psat);
    state.d :=dewDensity(sat);
    state.h :=dewEnthalpy(sat);
    else
      assert(sat.psat<MoMoLib.R134a.R134aData.data.FPCRIT, "function setDewState is only valid in two-phase regime");
    end if;
    annotation ( Documentation(info="<html>
<p>This function shall be used in order to calculate the thermodynamic state record for the vapor phase boundary. It requires the saturation record as input which can be determined by both functions setSat_p and setSat_T:
</p>
<p>
Example:
<pre>
    Medium.AbsolutePressure p=3e5;
    // Viscosity on the vapor phase boundary
    Modelica.SIunits.DynamicViscosity eta_vap;

    equation

    eta_vap = Medium.DynamicViscosity(Medium.setBubbleState(Medium.setSat_p(p)));
</pre>

<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end setDewState;


  redeclare function density_ph
  "density as function of pressure and specific enthalpy"
    extends Modelica.Icons.Function;
    input AbsolutePressure p "pressure";
    input SpecificEnthalpy h "specific enthalpy";
    input Integer phase = 0 "2 for two-phase, 1 for one-phase, 0 if not known";
    output Density d "density";

  algorithm
    d := rho_props_ph(p, h,
      derivsOf_ph(p, h, getPhase_ph(p,h)));

    annotation(Inline=true,
    Documentation(info="<html>
<p> This function calculates the density of R134a from the state variables p (absolute pressure) and h (specific enthalpy). The density is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>

<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)d-Diagram-R134a.png\"/> </p>

</html>"));
  end density_ph;


  redeclare function extends density
  "density as function of pressure and specific enthalpy | use setState_phX function for input"

  algorithm
   d:= density_ph(state.p, state.h);

    annotation(Inline=true,
      Documentation(info="<html>
<p> This function calculates the density of R134a from the state record (e.g., use setState_phX function for input). The density is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>

<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)d-Diagram-R134a.png\"/> </p>
</html>"));
  end density;


  redeclare function temperature_ph
  "temperature as function of pressure and specific enthalpy"
    extends Modelica.Icons.Function;
    input AbsolutePressure p "pressure";
    input SpecificEnthalpy h "specific enthalpy";
    input Integer phase =   0
    "2 for two-phase, 1 for one-phase, 0 if not known";
    output Temperature T "temperature";

  algorithm
    T := T_props_ph(p, h,
      derivsOf_ph(p, h, getPhase_ph(p,h)));

    annotation(Inline=true,
    Documentation(info="<html>
<p> This function calculates the Kelvin temperature of R134a from the state variables p (absolute pressure) and h (specific enthalpy). The temperature is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>

<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)h-Diagram-R134a.png\"/> </p>


</html>"));
  end temperature_ph;


  redeclare function extends temperature
  "temperature as function of pressure and specific enthalpy | use setState_phX function for input"

  algorithm
   T:= temperature_ph(state.p, state.h);

    annotation(Inline=true,
      Documentation(info="<html>
<p> This function calculates the Kelvin temperature of R134a from the state record (e.g., use setState_phX function for input). The temperature is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)h-Diagram-R134a.png\"/> </p>
</html>"));
  end temperature;


  redeclare function extends pressure "pressure w.r.t. thermodynamic state"
  algorithm
    p := state.p;
    annotation (
      Inline=true,
      Documentation(info="<html>
<p>This function is included for the sake of completness.</p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)h-Diagram-R134a.png\"/> </p>
</html>"));
  end pressure;


  redeclare function extends specificInternalEnergy
  "specific internal energy  w.r.t. thermodynamic state"
  algorithm
    u := specificEnthalpy(state) - pressure(state)/density(state);

    annotation (Inline = true,  Documentation(info="<html>
<p> This function calculates the specific internal energy of R134a from the state record (e.g., use setState_phX function for input). The specific internal energy is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)u-Diagram-R134a.png\"/> </p>

</html>"));
  end specificInternalEnergy;


  redeclare function extends specificEnthalpy
  "specific enthalpy w.r.t. thermodynamic state | use setState_phX function for input"

  algorithm
   h:=state.h;

    annotation (Inline = true,  Documentation(info="<html>
<p>This function is included for the sake of completness.</p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)h-Diagram-R134a.png\"/> </p>
</html>"));
  end specificEnthalpy;


  redeclare function extends specificEntropy
  "specific entropy w.r.t. thermodynamic state | use setState_phX function for input if necessary"

protected
    Common.HelmholtzDerivs f "helmholtz derivatives";
    Common.PhaseBoundaryProperties liq "properties on liquid phase boundary";
    SaturationProperties sat "saturation temperature and pressure";
    Common.PhaseBoundaryProperties vap "properties on vapor phase boundary";

    Modelica.SIunits.MassFraction x "vapor quality";

  algorithm
    if getPhase_ph(state.p, state.h)==2 then
      sat.psat :=state.p;
      sat.Tsat :=saturationTemperature(state.p);
      liq := R134a_liqofdT(T=state.T);
      vap := R134a_vapofdT(T=state.T);
      x := if liq.h<>vap.h then (state.h-liq.h)/(vap.h - liq.h) else if state.h >= vap.h then 1 else 0;
      s := liq.s + x*(vap.s - liq.s);
    else
      f := f_R134a(state.d, state.T);
      s := R134aData.R*(f.tau*f.ftau - f.f);
    end if;

    annotation ( Documentation(info="<html>
<p> This function calculates the specific entropy of R134a from the state record (e.g., use setState_phX function for input). The specific entropy is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)s-Diagram-R134a.png\"/> </p>
</html>"));
  end specificEntropy;


  redeclare function extends saturationTemperature
  "saturation temperature in two-phase region"

protected
    constant Real T_coef[:,:] = R134aData.Tcoef
    "coefficients of cubic spline for Tsat(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := p/R134aData.data.FPCRIT;
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    T := Common.CubicSplineEval(localx, T_coef[int, :]);

  // annotation(smoothOrder=5);
     annotation (derivative=saturationTemperature_der_p,
    Documentation(info="<html>
<p>This function calculates the saturation temperature of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)Tsat-Diagram-R134a.png\"/> </p>
</html>"));
  end saturationTemperature;


  redeclare function extends saturationTemperature_derp
  "derivative of saturation temperature in two-phase region"

protected
    constant Real T_coef[:,:] = R134aData.Tcoef
    "coefficients of cubic spline for Tsat(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := p/R134aData.data.FPCRIT;
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    dTp := Common.CubicSplineDerEval(localx,
      T_coef[int, :])/R134aData.data.FPCRIT;

    annotation ( Documentation(info="<html>
<p>This function calculates the derivative of saturation temperature of R134a with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://MoMoLib.R134a.R134a_ph.saturationTemperature\"> saturatuionTemperature</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end saturationTemperature_derp;


  redeclare function extends bubbleDensity
  "density of liquid phase w.r.t saturation pressure | use setSat_p function for input"

protected
    constant Real dl_coef[:,:] = R134aData.dlcoef
    "coefficients of cubic spline for d_liq(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    dl := R134aData.data.FDCRIT*
      Common.CubicSplineEval(localx, dl_coef[int, 1:4]);

  // annotation(smoothOrder=5);
     annotation (derivative=dBubbleDensity_dPressure_der_sat,
    Documentation(info="<html>
<p>This function calculates the liquid phase density of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end bubbleDensity;


  redeclare function extends dBubbleDensity_dPressure
  "derivative of liquid density in two-phase region w.r.t pressure"

protected
    constant Real dl_coef[:,:] = R134aData.dlcoef
    "coefficients of cubic spline for d_liq(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    ddldp := R134aData.data.FDCRIT*
      Common.CubicSplineDerEval(localx, dl_coef[int, 1:4])/R134aData.data.FPCRIT;

    annotation ( Documentation(info="<html>
<p>This function calculates the derivative of liquid density of R134a in the two-phase region with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://MoMoLib.R134a.R134a_ph.bubbleDensity\"> bubbleDensity</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end dBubbleDensity_dPressure;


  redeclare function extends dewDensity
  "density of vapor phase w.r.t saturation pressure | use setSat_p function for input"

protected
    constant Real dv_coef[:,:] = R134aData.dvcoef
    "coefficients of cubic spline for d_vap(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    dv := R134aData.data.FDCRIT*
      Common.CubicSplineEval(localx, dv_coef[int, 1:4]);

  // annotation(smoothOrder=5);
     annotation (derivative=dDewDensity_dPressure_der_sat,
      Documentation(info="<html>
<p>This function calculates the vapor phase density of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end dewDensity;


  redeclare function extends dDewDensity_dPressure
  "derivative of vapor density in two-phase region w.r.t pressure"

protected
    constant Real dv_coef[:,:] = R134aData.dvcoef
    "coefficients of cubic spline for d_vap(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    ddvdp := R134aData.data.FDCRIT*
      Common.CubicSplineDerEval(localx, dv_coef[int, 1:4])/R134aData.data.FPCRIT;

    annotation ( Documentation(info="<html>
<p>This function calculates the derivative of vapor density of R134a in two-phase region with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://MoMoLib.R134a.R134a_ph.dewDensity\"> dewDensity</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end dDewDensity_dPressure;


  redeclare function extends bubbleEnthalpy
  "specific enthalpy of liquid phase w.r.t saturation pressure | use setSat_p function for input"

protected
    constant Real hl_coef[:,:] = R134aData.hlcoef
    "coefficients of cubic spline for h_liq(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    hl := R134aData.data.HCRIT*
      Common.CubicSplineEval(localx, hl_coef[int, 1:4]);

  // annotation(smoothOrder=5);
     annotation (derivative=dBubbleEnthalpy_dPressure_der_sat,
      Documentation(info="<html>
<p>This function calculates the liquid phase enthalpy of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end bubbleEnthalpy;


  redeclare function extends dBubbleEnthalpy_dPressure
  "derivative of liquid specific enthalpy in two-phase region w.r.t pressure"

protected
    constant Real hl_coef[:,:] = R134aData.hlcoef
    "coefficients of cubic spline for h_liq(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    dhldp := R134aData.data.HCRIT*
      Common.CubicSplineDerEval(localx, hl_coef[int, 1:4])/R134aData.data.FPCRIT;

    annotation ( Documentation(info="<html>
<p>This function calculates the derivative of liquid enthalpy of R134a with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://MoMoLib.R134a.R134a_ph.bubbleEnthalpy\"> bubbleEnthalpy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end dBubbleEnthalpy_dPressure;


  redeclare function extends dewEnthalpy
  "specific enthalpy of vapor phase w.r.t saturation pressure | use setSat_p function for input"

protected
    constant Real hv_coef[:,:] = R134aData.hvcoef
    "coefficients of cubic spline for h_vap(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    hv := R134aData.data.HCRIT*
      Common.CubicSplineEval(localx, hv_coef[int, 1:4]);

  // annotation(smoothOrder=5);
     annotation (derivative=dDewEnthalpy_dPressure_der_sat,
      Documentation(info="<html>
<p>This function calculates the vapor phase enthalpy of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end dewEnthalpy;


  redeclare function extends dDewEnthalpy_dPressure
  "derivative of vapor specific enthalpy in two-phase region w.r.t pressure"

protected
    constant Real hv_coef[:,:] = R134aData.hvcoef
    "coefficients of cubic spline for h_vap(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    dhvdp := R134aData.data.HCRIT*
      Common.CubicSplineDerEval(localx, hv_coef[int, 1:4])/R134aData.data.FPCRIT;

    annotation ( Documentation(info="<html>
<p>This function calculates the derivative of vapor enthalpy of R134a in the two-phase region with regard to the state variable p (absolute pressure). The non-derivative function is <a href=\"modelica://MoMoLib.R134a.R134a_ph.dewEnthalpy\"> dewEnthalpy</a>.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end dDewEnthalpy_dPressure;


  redeclare function extends dewEntropy
  "specific entropy of vapor phase w.r.t saturation pressure | use setSat_p function for input"

protected
    constant Real sv_coef[:,:] = R134aData.svcoef
    "coefficients of cubic spline for s_vap(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    sv := R134aData.data.SCRIT*
      Common.CubicSplineEval(localx, sv_coef[int, 1:4]);

  // annotation(smoothOrder=5);
     annotation (derivative=dDewEntropy_dPressure_der_sat,
      Documentation(info="<html>
<p>This function calculates the vapor phase entropy of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end dewEntropy;


  redeclare function extends bubbleEntropy
  "specific entropy of liquid phase w.r.t saturation pressure | use setSat_p function for input"
protected
    constant Real sl_coef[:,:] = R134aData.slcoef
    "coefficients of cubic spline for s_liq(p)";
    constant Real p_breaks[:] = R134aData.pbreaks
    "grid points of reduced pressure";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    pred := min(sat.psat/R134aData.data.FPCRIT, 1.0);
    (int,error) := Common.FindInterval(pred,
      p_breaks);
    localx := pred - p_breaks[int];
    sl := R134aData.data.SCRIT*
      Common.CubicSplineEval(localx, sl_coef[int, 1:4]);

  // annotation(smoothOrder=5);
     annotation (derivative=dBubbleEntropy_dPressure_der_sat,
      Documentation(info="<html>
<p>This function calculates the liquid phase entropy of R134a from the state variable p (absolute pressure). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from
the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
</html>"));
  end bubbleEntropy;


  redeclare function extends saturationPressure
  "saturation pressure w.r.t. temperature"

protected
    constant Real pt_coef[:,:] = R134aData.ptcoef
    "coefficients of cubic spline for psat(T)";
    constant Real T_breaks[:] = R134aData.Tbreaks
    "grid points of reduced pressure";
    Real Tred "reduced temperature";
    Integer int "interval number";
    Integer error "interval for spline interpolation not found";
    Real pred "reduced pressure";
    Real localx "oordinate of local spline";
  algorithm
    Tred := T/R134aData.data.FTCRIT;
    (int,error) := Common.FindInterval(Tred, T_breaks);
    localx := Tred - T_breaks[int];
    p := R134aData.data.FPCRIT*Common.CubicSplineEval(localx, pt_coef[int, :]);

    annotation(smoothOrder=5, Documentation(info="<html>
<p>This function calculates the saturation pressure of R134a from the state variable T (temperature). It is modelled by cubic splines which are fitted with non-equidistant grid points derived from the fundamental equation of state of Tillner-Roth and Baehr (1994) and the Maxwell criteria.
</p>
<h3> Restrictions</h3>
<p>It is only valid in the two-phase region (i.e., p<sub>triple</sub> &le; p &le; p<sub>crit</sub> ).
</p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)Tsat-Diagram-R134a.png\"/> </p>
</html>"));
  end saturationPressure;


  redeclare function extends specificHeatCapacityCp
  "specific heat capacity at constant pressure | turns inifity in two-phase region! | use setState_phX function for input "

protected
    Common.HelmholtzDerivs f "helmholtz derivatives";

  algorithm
   f := f_R134a(state.d, state.T);
   cp := if getPhase_ph(state.p, state.h) == 2 then 0 else R134aData.R*(-f.tau*f.tau*f.ftautau + (f.delta*f.fdelta - f.delta*f.tau
      *f.fdeltatau)^2/(2*f.delta*f.fdelta + f.delta*f.delta*f.fdeltadelta));

    annotation ( Documentation(info="<html>
<p> This function calculates the specific heat capacity of R134a at <b>constant pressure</b> from the state record (e.g., use setState_phX function for input). The specific heat capacity is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)cp-Diagram-R134a.png\"/> </p>
</html>"));
  end specificHeatCapacityCp;


  redeclare function extends specificHeatCapacityCv
  "specific heat capacity at constant volume | use setState_phX function for input"

protected
    Common.HelmholtzDerivs f "helmholtz derivatives";
    Common.PhaseBoundaryProperties liq "properties on liquid phase boundary";
    SaturationProperties sat "saturation temperature and pressure";
    Common.PhaseBoundaryProperties vap "properties on vapor phase boundary";

    Modelica.SIunits.MassFraction x "vapor quality";

  algorithm
    if getPhase_ph(state.p, state.h)==2 then
    sat.psat :=state.p;
    sat.Tsat :=saturationTemperature(state.p);
     liq := R134a_liqofdT(T=state.T);
     vap := R134a_vapofdT(T=state.T);
     x := if liq.h<>vap.h then (state.h-liq.h)/(vap.h - liq.h) else if state.h >= vap.h then 1 else 0;
     cv := Common.cv2Phase(liq=liq, vap=vap, x=x, T=state.T, p=state.p);
    else
     f := f_R134a(state.d, state.T);
     cv :=  R134aData.R*(-f.tau*f.tau*f.ftautau);
    end if;

    annotation ( Documentation(info="<html>
<p> This function calculates the specific heat capacity of R134a at <b>constant volume</b> from the state record (e.g., use setState_phX function for input). The specific heat capacity is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<p>
Please note, that the function can also be called in the two-phase region, but the output is not continuous for a phase transition (see Tillner-Roth and Baehr, 1994). Values in two-phase region are considerably higher than in one-phase domain. The following figure just shows one-phase properties.
</p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)cv-Diagram-R134a.png\"/> </p>
</html>"));
  end specificHeatCapacityCv;


  redeclare function extends dynamicViscosity
  "dynamic viscosity w.r.t. temperature and density | use setState_phX function for input"

protected
    Real omega_log "log of collision integral";
    Real omega "collision integral";

    constant Real K=0.021357 "Constant for low density term eta_star";
    constant Modelica.SIunits.Length sigma=0.50647e-09 "Hard-sphere diameter";
    constant Modelica.SIunits.Temperature epsilon_k=288.82 "empirical factor";
    constant Real a[5]={2.218816e-01,-5.079322e-01,1.285776e-01,-8.328165e-02,-2.713173e-02}
    "Coefficients for term of collision integral";
    constant Real b[13]={-1.7999496,4.6692621e+01,-5.3460794e+02,3.3604074e+03,
        -1.3019164e+04,3.3414230e+04,-5.8711743e+04,7.1426686e+04,-5.9834012e+04,
        3.3652741e+04,-1.2027350e+04,2.4348205e+03,-2.0807957e+02}
    "Coefficients for term of 2nd viscosity virial coefficient";
    constant Real c[6]={-0.331249e-01,-0.468509e-03,0.156983,3.073830,-0.306398,
        0.215221} "Coefficients for term of residiual viscosity";
    constant Real DCRIT_mol=4.9788 "critical density in mol/l";
    Real d_red "reduced density";
    Real B_eta "2nd visosity virial coeff.";
    Real E "temperature contribution to residual viscosity";
    Real eta_res "residual part";
    Real eta_star "dilute part";

  algorithm
    omega_log := 0;
    for i in 1:5 loop
      omega_log := omega_log + a[i]*(Modelica.Math.log(state.T/
        epsilon_k))^(i - 1);
    end for;
    omega := Modelica.Math.exp(omega_log);
    eta_star := (K*sqrt(R134aData.data.MM*1000))/((sigma*1e+09)^2*omega)*
      sqrt(state.T);

    d_red := state.d/R134aData.data.DCRIT;
    E := 1 + c[3]*state.T/R134aData.data.TCRIT;
    B_eta := 0;
    for i in 1:13 loop
      B_eta := B_eta + b[i]*(sqrt(state.T/epsilon_k))^(i - 1);
    end for;
    B_eta := B_eta*602.2137*(sigma)^3*DCRIT_mol;

    eta_res := (c[1] + B_eta*eta_star)*d_red + c[2]*d_red^2 +
      (c[5] + c[6]*d_red)/(c[4]*E - d_red) - c[5]/(
      c[4]*E);

    eta := (eta_star + eta_res*1e3)*1e-06;

    annotation ( Documentation(info="<html>
<p> This function calculates the dynamic viscosity of R134a from the state record (e.g., use setState_phX function for input). The dynamic viscosity is modelled by the corresponding states method of Klein, McLinden and Laesecke (1997). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
<h3>References</h3>
<dl><dt>Klein, McLinden and Laesecke: </dt>
<dd><b>An improved extended corresponding states method for estimation of visocsity of pure refrigerants and mixtures</b>.
Int. J. Refrig., Vol. 20, No.3, pp. 208-217, 1997.</dd>
</dl>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)eta-Diagram-R134a.png\"/> </p>
</html>"));
  end dynamicViscosity;


  redeclare function extends thermalConductivity
  "thermal conductivity w.r.t. thermodynamic state | use setState_phX function for input"

protected
    Common.HelmholtzDerivs f "helmholtz derivatives";
    Common.HelmholtzDerivs f_ref "helmholtz derivatives for reference state";
    Modelica.SIunits.ThermalConductivity lambda_dg
    "dilute gas contribution to lambda";
    R134aData.CoeffsThermalConductivity coeff
    "coefficients of thermal conductivity model";
    Modelica.SIunits.ThermalConductivity lambda_reduced "reduced lambda";
    Modelica.SIunits.ThermalConductivity lambda_crit
    "enhancement of lambda in the critical region";
    Modelica.SIunits.ThermalConductivity chi_star "correlation length";
    Modelica.SIunits.ThermalConductivity chi_star_ref "correlation length";
    Modelica.SIunits.ThermalConductivity delta_chi "chi_star - chi_star_ref";
    Real rho_molar "molar density [mol/l]";
    Real dddp "derivative of density w.r.t. pressure";
    Real dddp_ref "derivative of density w.r.t. pressure for reference state";
    Modelica.SIunits.Length xi "correlation length";
    Modelica.SIunits.SpecificHeatCapacity cp
    "specific heat capacity at constant pressure";
    Modelica.SIunits.SpecificHeatCapacity cv
    "specific heat capacity at constant volume";
    Modelica.SIunits.DynamicViscosity eta "dynamic viscosity";
    Modelica.SIunits.ThermalConductivity omega "crossover function";
    Modelica.SIunits.ThermalConductivity omega_0 "crossover function";

  algorithm
    f:= f_R134a(state.d, state.T);
    f_ref:= f_R134a(state.d, coeff.T_ref);

    // dilute gase contribution
    lambda_dg := coeff.a_0 + coeff.a_1*state.T;

    // reduced lambda
    rho_molar := state.d/(R134aData.data.MM*1000);
    // [mol/L]
    lambda_reduced := 0;
    for i in 1:4 loop
      lambda_reduced := lambda_reduced + coeff.b[i]*(rho_molar/coeff.rho_crit)
        ^i;
    end for;
    lambda_reduced := lambda_reduced*coeff.lambda_reducing;

    // critical lambda
    dddp := 1/(f.R*f.T*f.delta*(2.0*f.fdelta + f.delta*f.fdeltadelta));
    chi_star := state.d*coeff.p_crit*1e3/((coeff.rho_crit*R134aData.data.MM*1000)^2)*
      dddp;
    dddp_ref := 1/(f_ref.R*f_ref.T*f_ref.delta*(2.0*f_ref.fdelta + f_ref.delta*f_ref.fdeltadelta));
    chi_star_ref := state.d*coeff.p_crit*1e3/((coeff.rho_crit*R134aData.data.MM*1000)^2)*
      dddp_ref;
    delta_chi := max(1e-10, (chi_star - chi_star_ref));
    xi := coeff.xi_0*(1/coeff.GAMMA*(delta_chi))^(coeff.nu/coeff.gamma);
    cp := R134aData.data.R*(-f.tau*f.tau*f.ftautau + (f.delta*f.fdelta - f.delta*f.tau*f.
       fdeltatau)^2/(2*f.delta*f.fdelta + f.delta*f.delta*f.fdeltadelta));
    eta := dynamicViscosity(state=state);
    cv := specificHeatCapacityCv(state=state);
    omega := 2/Modelica.Constants.pi*(((cp - cv)/cp)*Modelica.Math.atan(coeff.q_d*xi) +
      cv/cp*coeff.q_d*xi);
    omega_0 := 2/Modelica.Constants.pi*(1 - Modelica.Math.exp(-1/((coeff.q_d*xi)^(-1) +
      1/3*(coeff.q_d*xi*coeff.rho_crit/(state.d/(R134aData.data.MM*1000)))^2)));
    lambda_crit := state.d*cp*(1.03*Modelica.Constants.k*state.T)/(6*Modelica.Constants.pi*eta*
      xi)*(omega - omega_0);

    // conclusion
    lambda := max(lambda_dg  + lambda_reduced + lambda_crit, 1e-8);

    annotation ( Documentation(info="<html>
<p> This function calculates the thermal conductivity of R134a from the state record (e.g., use setState_phX function for input). The thermal conductivity is modelled by the corresponding states model of McLinden, Klein. and Perkins (2000). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
<h3>References</h3>
<dl><dt>McLinden, Klein. and Perkins: </dt>
<dd><b>An extended corresponding states model for the thermal conductivity
of refrigerants and refrigerant mixtures</b>.
Int. J. Refrig., 23 (2000) 43-63.</dd>
</dl>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)lambda-Diagram-R134a.png\"/> </p>
</html>"));
  end thermalConductivity;


  redeclare function extends surfaceTension
  "surface tension as a function of temperature (below critical point)"

protected
    Real tau "reduced temperatur";
    R134aData.CoeffsSurfaceTension coeff "polynomial coefficients";
    Modelica.SIunits.Temperature Tc=374.21 "critical temperature";

  algorithm
    if sat.Tsat > Tc then
      tau := 0.0;
      sigma := 0.0;
    else
      tau := 1.0 - sat.Tsat/Tc;
      sigma := 0.0;
      for i in 1:coeff.nterm loop
        sigma := sigma + coeff.sigmak[i]*tau^coeff.sigexp[i];
      end for;
    end if;

     annotation ( Documentation(info="<html>
<p> This function calculates the surface tension of R134a from the saturation record (e.g., use setSat_T function for input). The property is modelled by an approach of Okada and Higashi (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in two-phase region.
</p>
<h3>References</h3>
<dl><dt>Okada and Higashi: </dt>
<dd><b>Surface tension correlation of HFC-134a and HCFC-123</b>.
Proceedings of the Joint Meeting of IIR Commissions B1, B2, E1, and E2, Padua, Italy, pp. 541-548, 1994.</dd>
</dl>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/Tsigma-Diagram-R134a.png\"/> </p>
</html>"));
  end surfaceTension;


  redeclare function extends velocityOfSound
  "velocity of sound w.r.t. thermodynamic state (only valid for one-phase)"
protected
    MoMoLib.R134a.Common.HelmholtzDerivs f "helmholtz derivatives";
  algorithm

    if getPhase_ph(state.p, state.h)==2 then
      a:=0;
      // assert(getPhase_ph(state.p, state.h)==1, "Function for velocity of sound is only valid for one-phase regime!");
    else
    f := f_R134a(state.d, state.T);
    a := abs(R134aData.data.R*state.T*(2*f.delta*f.fdelta + f.delta*f.delta*f.
      fdeltadelta - ((f.delta*f.fdelta - f.delta*f.tau*f.fdeltatau)*(f.delta*f.
      fdelta - f.delta*f.tau*f.fdeltatau))/(f.tau*f.tau*f.ftautau)))^0.5;
    end if;
    annotation ( Documentation(info="<html>
<p> This function calculates the velocity of sound of R134a from the state record (e.g., use setState_phX function for input). The velocity of sound is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
<p><img src=\"modelica://MoMoLib/R134a/pictures/R134a/log(p)a-Diagram-R134a.png\"/> </p>
</html>"));
  end velocityOfSound;


  redeclare function extends isothermalCompressibility
  "isothermal compressibility w.r.t. thermodynamic state (only valid for one-phase)"
protected
    MoMoLib.R134a.Common.HelmholtzDerivs f "helmholtz derivatives";
  algorithm
    if getPhase_ph(state.p, state.h)==2 then
    kappa:=0;
    // assert(getPhase_ph(state.p, state.h)==1, "Function for isothermal compressibility is only valid for one-phase regime!");
    else
    f := f_R134a(state.d, state.T);
    kappa := 1/(state.d*R134aData.data.R*state.T*f.delta*(2.0*f.fdelta + f.delta*f.fdeltadelta));
    end if;
    annotation ( Documentation(info="<html>
<p> This function calculates the isothermal compressibility of R134a from the state record (e.g., use setState_phX function for input). The isothermal compressibility is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
</html>"));
  end isothermalCompressibility;


  redeclare function extends isobaricExpansionCoefficient
  "isobaric expansion coefficient w.r.t. thermodynamic state (only valid for one-phase)"
protected
    MoMoLib.R134a.Common.HelmholtzDerivs f "helmholtz derivatives";
  algorithm
    if getPhase_ph(state.p, state.h)==2 then
    beta:=0;
    // assert(getPhase_ph(state.p, state.h)==1, "Function for isobaric expansion coefficient is only valid for one-phase regime!");
    else
    f := f_R134a(state.d, state.T);
    beta:= R134aData.data.R*state.d*f.delta*(f.fdelta - f.tau*f.fdeltatau)*isothermalCompressibility(state);
    end if;
    annotation ( Documentation(info="<html>
<p> This function calculates the isobaric expansion coefficient of R134a from the state record (e.g., use setState_phX function for input). The isobaric expansion coefficient is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
</html>"));
  end isobaricExpansionCoefficient;


  redeclare function extends isentropicExponent
  "isentropic exponent gamma w.r.t. thermodynamic state | not defined in two-phase region | use setState_phX function for input"

  algorithm
     gamma := density(state)/(if pressure(state)>1e-6 then pressure(state) else 1e-6)*velocityOfSound(state)^2;

    annotation(Inline=true,
      Documentation(info="<html>
<p> This function calculates the isentropic exponent of R134a from the state record (e.g., use setState_phX function for input). The isentropic exponent is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). </p>
<h3> Restrictions</h3>
<p>This property is only defined in one-phase region.
</p>
</html>"));
  end isentropicExponent;


  redeclare function extends specificGibbsEnergy
  "specific gibbs energy w.r.t. thermodynamic state"
  algorithm
   g:= state.h - state.T*specificEntropy(state);
   annotation ( Documentation(info="<html>
<p> This function calculates the specific Gibbs energy of R134a from the state record (e.g., use setState_phX function for input). The isentropic exponent is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994).
</p>
</html>"));
  end specificGibbsEnergy;


redeclare function extends specificHelmholtzEnergy
  "helmholtz energy w.r.t. thermodynamic state"
algorithm
  f := state.h - state.p/state.d - state.T*specificEntropy(state);
     annotation ( Documentation(info="<html>
<p> This function calculates the specific Helmholtz energy of R134a from the state record (e.g., use setState_phX function for input). The helmholtz energy is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994).
</p>
</html>"));
end specificHelmholtzEnergy;


  redeclare function extends density_derh_p
  "density derivative by specific enthalpy | use setState_phX function for input"

protected
     Common.InverseDerivatives_rhoT derivs
    "inverse derivatives for density and temperature";

  algorithm
    derivs := derivsOf_ph(
      state.p,
      state.h,
      getPhase_ph(p=state.p,h=state.h));
    ddhp := if getPhase_ph(p=state.p,h=state.h)==2 then -state.d*state.d/(derivs.dpT*state.T) else -state.d*state.d*derivs.pt/(state.d*state.d*derivs.pd*derivs.cv + state.T*derivs.pt^2);

    annotation ( Documentation(info="<html>
<p> This function calculates the density derivative w.r.t. specific enthalpy at constant pressure of R134a (e.g., use setState_phX function for input). The derivative is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). It can be used for manual state transformations (e.g. from density to specific enthalpy).
</p>
</html>"));
  end density_derh_p;


  redeclare function extends density_derp_h
  "density derivative by pressure | use setState_phX function for input"

protected
     Common.InverseDerivatives_rhoT derivs
    "inverse derivatives for density and temperature";

  algorithm
    derivs := derivsOf_ph(
      state.p,
      state.h,
      getPhase_ph(p=state.p,h=state.h));
    ddph := if getPhase_ph(p=state.p,h=state.h)==2 then state.d*(state.d*derivs.cv/derivs.dpT + 1)/(derivs.dpT*state.T) else (state.d*(derivs.cv*state.d+derivs.pt))/(state.d*state.d*derivs.pd*derivs.cv + state.T*derivs.pt^2);

    annotation ( Documentation(info="<html>
<p> This function calculates the density derivative w.r.t. absolute pressure at constant specific enthalpy of R134a (e.g., use setState_phX function for input). The derivative is modelled by the fundamental equation of state of Tillner-Roth and Baehr (1994). It can be used for manual state transformations (e.g. from density to pressure).
</p>
</html>"));
  end density_derp_h;


  redeclare function extends isentropicEnthalpy
  "isentropic enthalpy of downstream pressure and upstream thermodynamic state (specific entropy)"
  algorithm
    h_is := specificEnthalpy_psX(p_downstream, specificEntropy(refState), reference_X);
       annotation ( Documentation(info="<html>
<p> This function calculates the specific enthalpy of R134a for an isentropic pressure change from refState.p to p_downstream (e.g., use setState_phX function for input of refState).
</p>
<p> The function can be used for instance to calculate an isentropic efficiency of a compressor or calculate the power consumption (obtained from the isentropic enthalpy) for a given efficiency.</p>
<p>
Example:
<pre>
     Medium.AbsolutePressure p_downstream=10e5;
     Medium.SpecificEnthalpy h_downstream=4.1e5;
     Medium.AbsolutePressure p_upstream=3e5;
     Medium.SpecificEnthalpy h_upstream=4.0e5;

     // Isentropic efficiency of a compressor:
     Real eta_is;

    equation

     h_is = isentropicEnthalpy(p_downstream, Medium.setState_phX(p_upstream, h_upstream));

     eta_is = (h_is-h_upstream)/(h_downstream - h_upstream);
</pre>
<h4>Restrictions</h4>
The isentropic efficiency function should not be applied in liquid region.
</p>
</html>"));
  end isentropicEnthalpy;


  redeclare function extends setSmoothState
  "smooth transition function between state_a and state_b"

protected
    Integer phase;

  algorithm
    phase := getPhase_ph(p=Modelica.Media.Common.smoothStep(x, state_a.p, state_b.p, x_small),
                         h=Modelica.Media.Common.smoothStep(x, state_a.h, state_b.h, x_small));
    state := ThermodynamicState(p=Modelica.Media.Common.smoothStep(x, state_a.p, state_b.p, x_small),
                                T=Modelica.Media.Common.smoothStep(x, state_a.T, state_b.T, x_small),
                                d=Modelica.Media.Common.smoothStep(x, state_a.d, state_b.d, x_small),
                                h=Modelica.Media.Common.smoothStep(x, state_a.h, state_b.h, x_small),
                                phase=phase);

  end setSmoothState;


  annotation (Documentation(info="<html>
<p>
Calculation of fluid properties for Tetrafluoroethane (R134a) in the fluid region of 0.0039 bar (Triple pressure) to 700 bar and 169.85 Kelvin (Triple temperature) to 455 Kelvin.
</p>
<p>
<h4>Restriction</font></h4>
</p>
<p>
The functions provided by this package shall be used inside of the restricted limits according to the referenced literature.
</p>
<ul>
 <li>
      <b> 0.0039 bar &le; p &le; 700 bar </b>
 </li>
 <li>
      <b> 169.85 Kelvin &le; T &le; 455 Kelvin  </i>
 </li>
 <li>
      <b> explicit for pressure and specific enthalpy </i>
 </li>
</ul>

<p><b>References</font></b> </p>
<dl><dt>Baehr, H.D. and Tillner-Roth, R.: </dt>
<dd><b>Thermodynamic Properties of Environmentally Acceptable Refrigerants -
Equations of State and Tables for Ammonia, R22, R134a, R152a, and R123</b>. Springer-Verlag, Berlin (Germany), 1994.</dd>
</dl>
<dl><dt>Klein, McLinden and Laesecke: </dt>
<dd><b>An improved extended corresponding states methode for estimation of visocsity of pure refrigerants and mixtures</b>.
Int. J. Refrig., Vol. 20, No.3, pp. 208-217, 1997.</dd>
</dl>
<dl><dt>McLinden, Klein. and Perkins: </dt>
<dd><b>An extended corresponding states model for the thermal conductivity
of refrigerants and refrigerant mixtures</b>.
Int. J. Refrig., 23 (2000) 43-63.</dd>
</dl>
<dl><dt>Okada and Higashi: </dt>
<dd><b>Surface tension correlation of HFC-134a and HCFC-123</b>.
Proceedings of the Joint Meeting of IIR Commissions B1, B2, E1, and E2, Padua, Italy, pp. 541-548, 1994.</dd>
</dl>
</html>"));
end R134a_ph;
